/*
 * defines.h - Define useful values and independent macros.
 */

#ifndef _DEFINES_H_
#define _DEFINES_H_


#define SCHAR_MIN   (-128)      /* minimum signed char value */
#define SCHAR_MAX     127       /* maximum signed char value */
#define UCHAR_MAX     0xff      /* maximum unsigned char value */

#define SHRT_MIN    (-32768)        /* minimum (signed) short value */
#define SHRT_MAX      32767         /* maximum (signed) short value */
#define USHRT_MAX     0xffff        /* maximum unsigned short value */

#ifndef INT32_MIN
#define INT32_MIN   (-2147483647 - 1) /* minimum (signed) int value */
#endif

#ifndef INT32_MAX
#define INT32_MAX     2147483647    /* maximum (signed) int value */
#endif

#ifndef UINT32_MAX
#define UINT32_MAX    0xffffffff    /* maximum unsigned int value */
#endif

#ifndef LONG_MIN
#define LONG_MIN    (-2147483647L - 1) /* minimum (signed) long value */
#endif

#ifndef LONG_MAX
#define LONG_MAX      2147483647L   /* maximum (signed) long value */
#endif

#ifndef ULONG_MAX
#define ULONG_MAX     0xffffffffUL  /* maximum unsigned long value */
#endif

#ifndef FLOAT_MAX 
#define FLOAT_MAX      3.402823466e+38F   /* maximum (signed) float value */
#endif


#define Zero          ( (float) 0.0 )
#define One           ( (float) 1.0 )
#ifndef M_PI
#define M_PI		  3.1415926535897932384626433832795
#endif
#define M_PIF		  ( (float) M_PI )
#ifndef M_2PI
#define M_2PI		  6.28318530717958647692528676655901
#endif
#ifndef M_SQRT2
#define M_SQRT2		  1.4142135623730950488016887242097
#endif
#ifndef M_INV_SQRT2
#define M_INV_SQRT2	  0.707106781186547524400844362104849
#endif

#define BITS_IN_INT32 32
#define BITS_IN_INT16 16
#define BITS_IN_INT8  8

#ifndef BITS_TO_BYTES
#define BITS_TO_BYTES(arg)		( ( arg ) >> 3 )
#endif

#ifndef BYTES_TO_BITS
#define BYTES_TO_BITS(arg)		( ( arg ) << 3 )
#endif

#ifndef SIZE_IN_BITS
#define SIZE_IN_BITS(arg)		( sizeof(arg) << 3 )
#endif

#ifndef MAX
#define MAX(a,b)      (((a) > (b)) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a,b)      (((a) < (b)) ? (a) : (b))
#endif

#ifndef ASM_LOG							
#define ASM_LOG( b_fact, x, lnx )				\
  __asm fld   b_fact							\
  __asm fld   x									\
  __asm fyl2x								    \
  __asm fstp  lnx
#endif




#ifndef FLOAT2SHORT_WHATEVER
#define FLOAT2SHORT_WHATEVER( val, RetVal16 )														\
{																									\
	__asm   fld     [val]                                       /* Take float number */				\
    __asm   fistp 	word ptr [RetVal16]                         /* Store it as a 16-bit integer */	\
}
#endif


#ifndef FLOAT2INT_WHATEVER
#define FLOAT2INT_WHATEVER( val, RetVal32 )														\
{																									\
	__asm   fld     [val]                                       /* Take float number */				\
    __asm   fistp 	dword ptr [RetVal32]                        /* Store it as a 32-bit integer */	\
}
#endif


#ifndef FLOAT2QUAD_WHATEVER
#define FLOAT2QUAD_WHATEVER( val, RetVal64 )														\
{																									\
	__asm   fld     [val]                                       /* Take float number */				\
    __asm   fistp 	qword ptr [RetVal64]                        /* Store it as a 64-bit integer */	\
}
#endif


/* Float2Int - Rounding of a floating point num to an integer. */
#ifndef FLOAT2INT_ROUND
#define FLOAT2INT_ROUND( val, RoundVal )																\
{																										\
    unsigned short  float_control_word_OLD;																\
	unsigned short  float_control_word_ROUND;															\
    long            long_value[2];																		\
	{ __asm   fnstcw	[float_control_word_OLD]				/* Store control-word */				\
    __asm   fld     [(val)]										/* Take float number*/					\
	__asm   mov		ax,0xF3FF									/* Mask to set rounding-mode to ROUND */\
	__asm   wait																						\
	__asm   and		ax,word ptr [float_control_word_OLD]		/* set rounding-mode to ROUND	 */		\
	__asm   mov		word ptr [float_control_word_ROUND],ax		/* Store value for ROUND		 */		\
	__asm   fldcw	[float_control_word_ROUND]					/* Load ROUND mode into FPU		 */		\
	__asm   fistp 	qword ptr [long_value]						/* Store it as a 64-bit integer	 */		\
	__asm   fldcw	[float_control_word_OLD]					/* Retrieve original Control-Word*/		\
	__asm   wait }												/* Make sure all's done			 */		\
    (*RoundVal) = long_value[0];																		\
}
#endif

#define ARRAY_NUM_ELEMENTS(__arr) ( sizeof(__arr) / sizeof((_arr)[0]) )

#ifndef FLOOR_LOG
#define FLOOR_LOG(p, nbits) ( ((int) (p)) & (-(1 << (nbits))) )
#define CEIL_LOG(p, nbits)  ( FLOOR_LOG(((int) (p))-1, (nbits)) +             \
							  (1 << (nbits)) )
#endif


#endif /* _GNR_DEFINES_H_ */
