/*----------------------------------------------------------------------------
 *
 * The information contained in this file is confidential and 
 * proprietary to Conexant Systems Inc.
 * No part of this file may be reproduced or distributed, in any form
 * or by any means for any purpose, without the express written
 * permission of Conexant Systems Inc.
 *
 * Copyright (C) 1996-2001 Conexant Systems Inc. 
 * All rights reserved.
 *
 *----------------------------------------------------------------------------
 *
 * File: general.h
 *
 * definitions and includes for memmgr, memops, strings, octopus, etc.
 *
 *----------------------------------------------------------------------------
 */
 
#ifndef __GENERAL_H__
#define __GENERAL_H__


#include "sysenv.h"
#if ( PROJECT == PRJ_HSF )
#include "hsf_types.h"
#endif


#ifdef __cplusplus

#if ( (USE_MEM_MGR == 1) && (OCT_MEM_MGR == 0) )
#include "memmgr.h"
#endif

#include "globals.h"


#if (OVERRIDE_NEW == 1)/* Allocation override for MemMgr: */

#if ( (OS_TYPE != OS_WIN_NT4) && (OS_TYPE != OS_WIN_WDM) )

CDecl( extern void*, operator new( unsigned int Size,
                                   char* FileName, int LineNumber ) );
CDecl( extern void*, operator new[]( unsigned int Size,
                                     char* FileName, int LineNumber ) );

#define new new( __FILE__, __LINE__ )

#endif

#endif

#endif /* __cplusplus */


#if ( (USE_MEM_MGR == 1) && (OCT_MEM_MGR == 1) )
#include "memmgr_ex.h"
#endif


/* In the Data Pumps the user should define DLL(OS_WIN_9X & UK_USER) */
/* The Application should define IMP_DLL to 1 (imported DLL) */
/* Amos should not define any of them. */

#if ( COMPILER == CMP_MSDEV )

#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) ) /* DLL code. */
#define DllPrefix   __declspec( dllexport )
#else
#if ( IMP_DLL == 1 ) /* Dll imported by app. */
#define DllPrefix   __declspec( dllimport )
#else
#define DllPrefix
#endif
#endif

#else /* Other compilers (gnu): no dll support via declspec! use .def files. */

#define DllPrefix

#endif


/* Some macros for interfacing with the octopus managers: */
#if ( OCT_SW_MODEM == 1 )

#define UpdateEvent(a)                                                        \
        gpStatMgrIf->UpdateStatus( ghStatMgrInstance,                         \
                                   ghStatMgrClient,                           \
                                   STATUS_RING_BURST,                         \
                                   (PVOID)a,                                  \
                                   TRUE );


#define CancelDPCTimer(a)                                                     \
        gpRTMgrIf->CancelAsyncCall( ghRTMgrInstance, ghRTMgrClient, (a) );

#define SetDPCTimer(a,b,c,d)                                                  \
{                                                                             \
    AsyncCallInfo.Type = ASYNC_CALL_TYPE_ONE_SHOT;                            \
    AsyncCallInfo.TimeOut = (b);                                              \
    AsyncCallInfo.pStack = NULL;                                              \
    AsyncCallInfo.StackSize = 0;                                              \
    AsyncCallInfo.pFuncAsyncCall = (c);                                       \
    AsyncCallInfo.TimerIRQL = DPC_IRQL;                                       \
    gpRTMgrIf->SetAsyncCall( ghRTMgrInstance, ghRTMgrClient,                  \
                             (a), (d), &AsyncCallInfo );                      \
} 

#define GetSystemTime(a)                                                      \
        gpRTMgrIf->Monitor( ghRTMgrInstance, RTMGR_MONITOR_TIME_STAMP, (a) );

#endif /* OCT_SW_MODEM */


/* Define allocation through New/Delete instead of MemMgr: */

#if ( USE_MEM_MGR == 1 )

#if ( OCT_MEM_MGR == 1 ) /* usage of octopus memory manager macros: */

#define MemAlloc(a,b,c) {                                                     \
    if ( NULL != gpMemMgrIf )                                                 \
    {                                                                         \
        (a) = (c *) gpMemMgrIf->Alloc( ghMemMgrInstance,                      \
                                       ghMemMgrClient,                        \
                                       __FILE__ ":" StringItVal(__LINE__),    \
                                       (b) * sizeof(c) );                     \
    }                                                                         \
    else                                                                      \
    {                                                                         \
        DPRINTF (("MemAlloc: gpMemMgrIf is NULL!"));                          \
        (a) = NULL;                                                           \
    }                                                                         \
}

#define MemFree(a) {                                                          \
    if ( NULL != gpMemMgrIf )                                                 \
    {                                                                         \
        gpMemMgrIf->Free( ghMemMgrInstance, ghMemMgrClient, (a) );            \
    }                                                                         \
    else                                                                      \
    {                                                                         \
        DPRINTF (("MemFree: gpMemMgrIf is NULL!"));                           \
    }                                                                         \
    (a) = NULL;                                                               \
}

#else /* Old style MEM_MGR: */

#define MemAlloc(a,b,c)  ( a = (c *)pMemMgr->Alloc((b)*sizeof(c), #a) )
#define MemFree(a)       ( pMemMgr->Free(a) )

#endif /* OCT_MEM_MGR */

#else /* No internal memory manager: */


#if ( UK_MODE == UK_USER )

#define MemAlloc(a,b,c) do                                                    \
{                                                                             \
    (a) = (c *) OsHeapAllocate( (b) * sizeof(c),  OS_NON_PAGED);              \
    if ( NULL != (a) )                                                        \
        OsMemSet(a,0,(b)*sizeof(c));                                            \
} while(0)

#define MemFree(a)                                                            \
{                                                                             \
    OsHeapFree(a);                                                                  \
    (a) = NULL;                                                               \
} while(0)

#elif ( OS_TYPE == OS_LINUX )

#include "missing_c.h"

#define MemAlloc(a,b,c) do                                                    \
{                                                                             \
    (a) = (c *) kernelmalloc( (b) * sizeof(c) );                              \
    OsMemSet(a,0,(b)*sizeof(c));                                                \
} while(0)

#define MemFree(a) kernelfree(a);


#else

/* Need to define here kernel (non-linux) implementation, when not using any
   memory manager. */

#endif

#endif /* USE_MEM_MGR */

 
/* defines for mem ops like memset, and string ops like strcmp in kernel: */
#if ( UK_MODE == UK_KERNEL )

#if ( OCT_SW_MODEM == 1 )

#include "osmemory_ex.h"
#define memset          OsMemSet
#define memcpy          OsMemCpy
#define memcmp          OsMemCmp
#define memmove         OsMemMove

#include "osstring_ex.h"
#define strcpy          OsStrCpy
#define strncpy         OsStrnCpy
#define strcat          OsStrCat
#define strncat         OsStrnCat
#define strcmp          OsStrCmp
#define strncmp         OsStrnCmp
#define toupper         OsToupper
#define tolower         OsTolower
#define isdigit         OsIsDigit
#define sprintf         OsSprintf
#define strlen          OsStrLen

#endif

#endif /* UK_MODE */

 
#include "std_defines.h"
 

#endif /* __GENERAL_H__ */
