/*************************************************************************

  File Name: DbgMgr_ex.h

  File Description:

  *************************************************************************/

#ifndef __DBG_MGR_EX_H__
#define __DBG_MGR_EX_H__
#include "intfctrl_ex.h"
#define	DBG_MGR_MAJOR_VERSION	0
#define	DBG_MGR_MINOR_VERSION	1

#if ( USE_PRAGMA_PACK == 1 )
#pragma pack(push,1)   /* Packing compatible with WinAC */
#endif

typedef enum {
	DBG_TOOL_TYPE_SCR,
	DBG_TOOL_TYPE_CONSOLE,
	DBG_TOOL_TYPE_DMP,
	DBG_TOOL_LAST
} DBG_TOOL_TYPE;


typedef enum
{
	DBGMGR_CONFIG_DBG_TOOL_INFO,	/*  DBG_TOOL_INFO*  		        */
	DBGMGR_CONFIG_PATH,
	DBGMGR_CONFIG_MAX_SIZE,
	DBGMGR_CONFIG_CLIENT_NAME,
	DBGMGR_CONFIG_LAST /* Dummy */
}DBGMGR_CONFIG_CODE;

typedef enum
{
	DBGMGR_MONITOR_LAST /* Dummy */
}DBGMGR_MONITOR_CODE;

typedef enum
{
	DBGMGR_CONTROL_DUMP_TO_FILE,				/* DBG_TOOL_INFO*               */
	DBGMGR_CONTROL_CLEAR_FILE,					/* HANDLE* hDbgTool             */
	DBGMGR_CONTROL_LAST /* Dummy */
}DBGMGR_CONTROL_CODE;

#define DBG_MGR_VERSION ((DBG_MGR_MAJOR_VERSION) << 16) | (DBG_MGR_MINOR_VERSION)

	
	/********************************************************************************
	Function Name: DebugMgrGetInterface.

	Function Description:	The function returns a pointer to the Debug manager
							interface structure that contains pointer to all the exported
							Debug manager functions.
	Arguments: None.
	
	Return Value: A pointer to the Debug manager interface functions structure.

	********************************************************************************/
	GLOBAL	PVOID	DebugMgrGetInterface(void);

typedef struct I_DBG_MGR_TAG{	
	/************ Common Interface functions. ************/
	/********************************************************************************
	
	Function Name: GetInterfaceVersion.

	Function Description:The function returns the interface compilation version.
						 The function should be called before any other function 
						 in the function table in order to verify that the calling 
						 code and the block code are compatible.
	
	Arguments: None.
	
	Return Value: The interface compilation version.

	********************************************************************************/
	UINT32		(*GetInterfaceVersion)(void);

	/********************************************************************************

	Function Name: Create.

	Function Description: The function creates a block instance.
	
	Arguments: MemMgrInterface - Access (function table & instance handle ) 
	                             to memory manager.
	
	Return Value: An handle to the created instance. (this handle will passed as an 
				  argument to all other functions to identify the block instance).
				  (in case of error it returns NULL).

	********************************************************************************/
	HANDLE		(*Create)(IN	PSYSTEM_INSTANCES_T	pSystemInstances);			

	/********************************************************************************

	Function Name: Destroy.

	Function Description: The function destroy the block instance identify by the hBlock
						  handle. After the destroying this handle becomes invalid. 
	
	Arguments: hblock - a handle to the block to destroy. (this handle was returned by 
						the create function when we created this block instance).
	
	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the block instance was successfully 
				  destroyed).

	********************************************************************************/
	COM_STATUS	(*Destroy)(IN HANDLE	hBlock);

	/********************************************************************************

	Function Name: Init.

	Function Description: The function initialize the block identified by the hBlock 
						  parameter.
	
	Arguments: a handle to the block to initialize.
	
	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the block instance was successfully
				  initialized).

	********************************************************************************/
	COM_STATUS	(*Init)(IN	HANDLE	hBlock);

	/********************************************************************************

	Function Name: Close.

	Function Description: The function closes the block identified by the hBlock handle.
						  After the instance was closed, its handle can be destroyed by 
						  the destroy function.
	
	Arguments: a handle to the block to close.
	
	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the block instance was successfully
				  closed).

	********************************************************************************/
	COM_STATUS	(*Close)(IN HANDLE	hBlock);		

	/********************************************************************************

	Function Name: Configure.

	Function Description: The function is used to update the block instance
						  configurations. Each ConfigCode associates with a specific
						  structure containing the configuration value/data.
						  The function can be called only after create and before init.
	
	Arguments: hBlock - block handle (identify of the block).
			   ConfigCode - enumerated code identifying the configuration type.
			   pConfig - pointer to a structure containing the configuration data.
	
	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the block instance was successfully
				  closed).

	********************************************************************************/
	COM_STATUS	(*Configure)(	IN HANDLE				hBlock,
								IN DBGMGR_CONFIG_CODE	ConfigCode,
								IN PVOID				pConfig);

	/********************************************************************************

	Function Name: Monitor.

	Function Description: The function is used to monitor a block instance.
						  Each MonitorCode is associates with a specific structure
						  containing the Monitoring information.
						  The function can be called only after init.
	
	Arguments: hBlock - block handle (identify of the block).
			   MonitorCode - enumerated code identifying the monitor type.
			   pMonitor - pointer to a structure that will be filled with the monitored data.
			   MonitorSize - size of the monitor structure.

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the block instance was successfully
				  monitored).

	********************************************************************************/
	COM_STATUS	(*Monitor)(	IN	HANDLE				hBlock,
							IN	DBGMGR_MONITOR_CODE	MonitorCode,
							OUT	PVOID				pMonitor);

	/********************************************************************************

	Function Name: Control.

	Function Description: The function is used to change the value of the block 
						  instance parameters. Each ControlCode is associates with 
						  a specific structure containing the control data/value.
						  The function can be called only after init.
	
	Arguments: hBlock - block handle (identify of the block).
			   ControlCode - enumerated code identifying the control type.
			   pControl - pointer to a structure containing the control data.
			   ControlSize - size of the control structure.
	
	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the block instance was successfully
				  controlled).

	********************************************************************************/
	COM_STATUS	(*Control)(	IN	HANDLE				hBlock,
							IN	DBGMGR_CONTROL_CODE	ControlCode,
							IN	PVOID				pControl);



	/************ Specific Interface functions. ************/

	/********************************************************************************
	
	Function Name: CreateDbgTool
	  
	Function Description: The function creates a specific type of debug tool instance.
						  The created debug tool can be use only with the Debug Manager
						  that created it.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   DebugToolType - enumerated code identifying the debug type.

	Return Value: a handle to the created debug tool. (in case of error it returns NULL).

	********************************************************************************/
	HANDLE	(*CreateDbgTool)(	IN	HANDLE			hDbgMgr,
								IN	DBG_TOOL_TYPE	DebugToolType);

	/********************************************************************************
	
	Function Name: DestroyDbgTool
	  
	Function Description: The function destroys a debug tool instance and the debug tool
						  instance handle becomes invalid.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the debug tool was successfully destroyed).

	********************************************************************************/
	COM_STATUS	(*DestroyDbgTool)(	IN	HANDLE	hDbgMgr,
									IN	HANDLE	hDbgTool);

	/********************************************************************************
	
	Function Name: InitDbgTool
	  
	Function Description: The function initializes the debug tool identified by the 
						  hdbgTool parameter.(it also allocates memory space for the 
						  debug tool, if needed).

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the debug tool was successfully initialized).

	********************************************************************************/
	COM_STATUS	(*InitDbgTool)(	IN	HANDLE	hDbgMgr,
								IN	HANDLE	hDbgTool);

	/********************************************************************************
	
	Function Name: CloseDbgTool
	  
	Function Description: The function closes the debug tool identified by the hDbgTool 
						  parameter. After the close the handle can be destroyed by the 
						  DestroyDbgTool function or reinitializaed by the InitdbgTool
						  function.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the debug tool was successfully closed).

	********************************************************************************/
	COM_STATUS	(*CloseDbgTool)(IN	HANDLE	hDbgMgr,
								IN	HANDLE	hDbgTool);	

	/********************************************************************************
	
	Function Name: ConfigureDbgTool
	  
	Function Description: The function is used to update the debug tool instance
						  configurations. Each ConfigCode associates with a specific
						  structure containing the configuration value/data.
						  The function can be called only after CreateDbgTool and 
						  before InitDbgTool.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.
			   ConfigCode - enumerated code identifying the configuration type.
			   pConfig - a pointer to a structure containing the configuration data.

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the debug tool was successfully configured).

	********************************************************************************/
	COM_STATUS	(*ConfigureDbgTool)(IN	HANDLE				hDbgMgr,
									IN	HANDLE				hDbgTool,
									IN	DBGMGR_CONFIG_CODE	ConfigCode,
									IN	PVOID				pConfig);

	/********************************************************************************
	
	Function Name: MonitorDbgToolt
	  
	Function Description: The function is used to monitor a debug tool instance.
						  Each MonitorCode is associates with a specific structure
						  containing the Monitoring information.
						  The function can be called only after InitDbgTool.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.
			   MonitorCode - enumerated code identifying the monitor type.
			   pMonitor - a pointer to a structure containing the monitor data.
			   MonitorSize - the size of the monitor structure (that pMonitor is pointing to).

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the debug tool was successfully monitored).

	********************************************************************************/
	COM_STATUS	(*MonitorDbgToolt)(	IN	HANDLE				hDbgMgr,
									IN	HANDLE				hDbgTool,
									IN	DBGMGR_MONITOR_CODE	MonitorCode,
									OUT	PVOID				pMonitor);


	/********************************************************************************
	
	Function Name: ControlDbgTool
	  
	Function Description: The function is used to change the value of the debug tool 
						  instance parameters. Each ControlCode is associates with 
						  a specific structure containing the control data/value.
						  The function can be called only after InitDbgTool.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.
			   ControlCode - enumerated code identifying the control type.
			   pControl - a pointer to a structure containing the control data.

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the debug tool was successfully controlled).

	********************************************************************************/
	COM_STATUS	(*ControlDbgTool)(	IN	HANDLE				hDbgMgr,
									IN	HANDLE				hDbgTool,
									IN	DBGMGR_CONTROL_CODE	ControlCode,
									IN	PVOID				pControl);

	/********************************************************************************
	
	Function Name: WriteDbgTool
	  
	Function Description: The function writes a given buffer to the file hDbgTool is 
						  referred to.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.
			   pBuff - is a pointer to a buffer containing the data that should be 
			   written to the debug tool.
			   Size - is the number of bytes written to the buffer.

	Return Value: the number of bytes that actually were written from the buffer to
				  the debug tool.

	********************************************************************************/
	int	(*WriteDbgTool)(IN	HANDLE	hDbgMgr,
						IN	HANDLE	hDbgTool,
						IN	PVOID	pBuff,
						IN	int		Size);

	/********************************************************************************
	
	Function Name: PrintfDbgTool
	  
	Function Description: The function writes data in a C "printf" format to the debug
						  tool identified by hDbgTool.

	Arguments: hDbgMgr - a handle to the Debug manager instance.
			   hDbgTool - a handle to the debug tool.
			   FormatString - is the data to be written to the debug tool in the same
			   format as the C function "printf".
			   ... - is for optional arguments.

	Return Value: COM_STATUS (COM_STATUS_SUCCESS if the printing has succeeded).

	********************************************************************************/
	COM_STATUS	(*PrintfDbgTool)(	IN	HANDLE	hDbgMgr,
									IN	HANDLE	hDbgTool,
									IN	PCHAR	FormatString,
										...);					/* ????? */

}I_DBG_MGR_T, *PI_DBG_MGR_T;

/* Debug manager access macros */



#if (defined SCR_DEBUG || defined SCR_RETAIL)
#define SCR_INSTANCE(Inst)	HANDLE Inst
#define SCR_OPEN(Inst, Path) { \
	Inst = ((PI_DBG_MGR_T)gpIDbgMgr)->CreateDbgTool(ghDbgMgrInstance, DBG_TOOL_TYPE_SCR); \
  	if ( NULL != Inst) \
	{ \
		((PI_DBG_MGR_T)gpIDbgMgr)->ConfigureDbgTool(ghDbgMgrInstance, Inst, DBGMGR_CONFIG_PATH, Path);\
		((PI_DBG_MGR_T)gpIDbgMgr)->InitDbgTool(ghDbgMgrInstance, Inst);\
	}\
}

#define SCR_CLOSE(Inst) {\
	if ( NULL != Inst) \
	{\
		((PI_DBG_MGR_T)gpIDbgMgr)->CloseDbgTool(ghDbgMgrInstance, Inst); \
		((PI_DBG_MGR_T)gpIDbgMgr)->DestroyDbgTool(ghDbgMgrInstance, Inst); \
		Inst = NULL; \
	}\
}

#define SCR_PRINT(num) SCR_PRINT##num
#define SCR_PRINT0(hScrTool,str)				((PI_DBG_MGR_T)gpIDbgMgr) -> PrintfDbgTool(\
												ghDbgMgrInstance, hScrTool, str)
#define SCR_PRINT1(hScrTool,str, p1)			((PI_DBG_MGR_T)gpIDbgMgr) -> PrintfDbgTool(\
												ghDbgMgrInstance, hScrTool, str, p1)
#define SCR_PRINT2(hScrTool,str, p1, p2)		((PI_DBG_MGR_T)gpIDbgMgr) -> PrintfDbgTool(\
												ghDbgMgrInstance, hScrTool, str, p1, p2)
#define SCR_PRINT3(hScrTool,str, p1, p2, p3)	((PI_DBG_MGR_T)gpIDbgMgr) -> PrintfDbgTool(\
												ghDbgMgrInstance, hScrTool, str, p1, p2, p3)
#endif





#ifdef SCR_DEBUG
#define SCR_DEBUG_INSTANCE(inst)		SCR_INSTANCE(inst);
#define SCR_EXTERN_DEBUG_INSTANCE(inst)	extern SCR_INSTANCE(inst);
#define SCR_DEBUG_OPEN(inst, Path)		SCR_OPEN(inst, Path)
#define SCR_DEBUG_CLOSE(inst)			SCR_CLOSE(inst)
#define SCR_DEBUG_PRINT(n)				SCR_PRINT(n)
#else
#define SCR_DEBUG_INSTANCE(inst)
#define SCR_EXTERN_DEBUG_INSTANCE(inst)	
#define SCR_DEBUG_OPEN(inst, Path)	
#define SCR_DEBUG_CLOSE(inst)	
#define SCR_DEBUG_PRINT(n)				SCR_DEBUG_PRINT##n
#define SCR_DEBUG_PRINT0(inst, str)
#define SCR_DEBUG_PRINT1(inst, str, p1)
#define SCR_DEBUG_PRINT2(inst, str, p1, p2)
#define SCR_DEBUG_PRINT3(inst, str, p1, p2, p3)

#endif

#ifdef SCR_RETAIL
#define SCR_RETAIL_INSTANCE(inst)		SCR_INSTANCE(inst);
#define SCR_EXTERN_RETAIL_INSTANCE(inst) extern SCR_INSTANCE(inst);
#define SCR_RETAIL_OPEN(inst, Path)		SCR_OPEN(inst, Path)	
#define SCR_RETAIL_CLOSE(inst)			SCR_CLOSE(inst)
#define SCR_RETAIL_PRINT(n)				SCR_PRINT(n)
#else
#define SCR_RETAIL_INSTANCE(inst)
#define SCR_EXTERN_RETAIL_INSTANCE(inst) 	
#define SCR_RETAIL_OPEN(inst, Path)	
#define SCR_RETAIL_CLOSE(inst)	
#define SCR_RETAIL_PRINT(n)				SCR_RETAIL_PRINT
#define SCR_RETAIL_PRINT0(inst, str)
#define SCR_RETAIL_PRINT1(inst, str, p1)
#define SCR_RETAIL_PRINT2(inst, str, p1, p2)
#define SCR_RETAIL_PRINT3(inst, str, p1, p2, p3)
#endif


#if ( USE_PRAGMA_PACK == 1 )
#pragma pack(pop)   /* Packing compatible with our VXDs */
#endif

#endif	/* __DBG_MGR_EX_H__ */
