/*************************************************************************

  File Name: devmgr_ex.h

  File Description:

  *************************************************************************/

#ifndef __DRV_MGR_E_H__
#define __DRV_MGR_E_H__

#include "comtypes.h"
#include "element_ex.h"
#include "pim.h"
#include "dm_stream.h"
#include "configcodes.h"

#if ( USE_PRAGMA_PACK == 1 )
#pragma pack(push,1)   // Packing compatible with WinAC
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#define	DRV_MGR_MAJOR_VERSION	0
#define	DRV_MGR_MINOR_VERSION	2

#define DRV_MGR_VERSION ((DRV_MGR_MAJOR_VERSION) << 16) | (DRV_MGR_MINOR_VERSION)

typedef enum 
{
	DRVMGR_CONFIG_DEV_NODE,		/*  UINT32* 	*/
	DRVMGR_CONFIG_LAST,			/* Dummy		*/
} DRVMGR_CONFIG_CODE;

typedef enum 
{
	DRVMGR_MONITOR_LAST,		 /* Dummy */
} DRVMGR_MONITOR_CODE;
	
typedef enum 
{
	DRVMGR_CONTROL_ENABLE_DRIVER_INTERRUPTS,	/* BOOL *bEnable.	*/
	DRVMGR_CONTROL_SLEEP,						/* NULL				*/
    DRVMGR_CONTROL_WAKEUP,						/* NULL				*/
	DRVMGR_CONTROL_LAST,						/* Dummy			*/
} DRVMGR_CONTROL_CODE;


// See DevMgr_CreateToneDetector
typedef enum {
	TONE_PRIMARY,				// 0x0
	TONE_ALTERNATE,
	TONE_ATBEL,
	TONE_ATV25,
	TONE_1100,
	TONE_2130,
	TONE_2312,
	TONE_2750,
	TONE_2912,
	VOICETONEACALLPROG,
	VOICETONEB_1100,
	VOICETONEC_1300,
	VOICETONEC_2100,
	VOICETONEB_2225,
	V8BTONE_2100,				// 0xE
	V8BTONE_2225,				// 0xF
	CI_CENTER_TONEB,
	CI_SIDE_TONEC,
	VOICEVIEW_TONEC_2912,
	VOICEVIEW_TONEB_2312,
	VOICEVIEW_TONEB_2750,
	VOICEVIEW_TONEA_2130,
	VOICEVIEW_TONEA_NOT_ST,
	TONEC_1650,
	CReSEG1_2002,
    TONE_1270,
	TONE_GEN_V25CALL,				/* TONEGEN_PARAMS*		*/
	TONE_GEN_CNG,					/* TONEGEN_PARAMS*				*/
	TONE_GEN_CED,					/* TONEGEN_PARAMS*				*/
	TONE_GEN_CI,						/* TONEGEN_PARAMS*				*/
    TONE_GEN_1650,
} eTONEDETECT_TYPE;




	
/*********************** Driver Manager Exported types :********************/

/********************************************************************************
Function Name: DevMgrGetInterface.

Function Description:Return pointer to Driver Manager interface.

rguments: None.

Return Value: Pointer to Driver interface function table.

********************************************************************************/
PVOID DevMgrGetInterface(void);

// See DevMgr_Configure function
typedef enum {
	DEVMGR_ASSIGN_HW_RES
}  eConfigureCode;

// See DevMgr_Monitor function
typedef enum {
	DEVMGR_TRAINING_EQM_SUM,				// AT#UD, AT#UG
	DEVMGR_DIGITAL_LOSS_ESTIMATE,			// go to connected state, V90, read digital loss
	DEVMGR_GET_TRANSMIT_OUTPUT_LEVEL,		// AT#UD
	DEVMGR_GET_TRAIN_FLEX_SKIP_V8,			// ???
	DEVMGR_HIGH_POWER_SERVER,				// LMON
	DEVMGR_RATEMASK,						// LMON
	DEVMGR_TX_BITRATE,
	DEVMGR_RX_BITRATE,
	DEVMGR_RX_BITRATE_EFF,					// K56
	DEVMGR_EQM,								// LMON, get EQM
	DEVMGR_RXAGC,							// LMON, get AGC gain word (to calculate receive level)
	DEVMGR_RX_CARRIER,						// AT#UD
	DEVMGR_SNR,								// signal to noise ratio
	DEVMGR_MODULATION,
	DEVMGR_TX_SYMBOLRATE,
	DEVMGR_RX_SYMBOLRATE,
	DEVMGR_RBSMASK,							// AT#UD, AT#UG
	DEVMGR_PADDETECT,						// AT#UD, AT#UG
	DEVMGR_RECEIVE_AGC_AUTO_GAIN,			// LMON, get auto gain
	DEVMGR_GET_DAA_STATE,					// see GetDAARelay function
	DEVMGR_STATUSMGR_CLIENT,				// Device Manager as a client of Status Manager
	DEVMGR_FLEX_CTLDOWNLOADVERSION
} eMonitorCode;

// See DevMgr_Control function
typedef enum {
	DEVMGR_ENABLE_FIFO,
	DEVMGR_DISABLE_FIFO,
	DEVMGR_ENABLE_COMPR_EQUALIZER,			// V8bis
	DEVMGR_DISABLE_COMPR_EQUALIZER,			// connect, if V23
	DEVMGR_SET_TRANSMIT_LEVEL,				// DPISetTransmitLevel
	DEVMGR_SET_CALL_MODE,					// TRUE-Originate, FALSE-Answer
	DEVMGR_CONTROLLER_MODE,					// fax, voice, EC, EC_NONE, etc.
	DEVMGR_CTRL_UPDATE_EC_ERR_CNT,			// parameter requested by LMON
	DEVMGR_CTRL_ATH_IN_PROGRESS,			// parameter requested by LMON
	DEVMGR_CTRL_CLOSE_CONNECTION,			// for ATH
	DEVMGR_SET_DAA_STATE,					// see SetDAARelay function
	DEVMGR_ACTIVATE_LMON,
	DEVMGR_DEACTIVATE_LMON
} eControlCode;


// See DevMgr_Control function + DEVMGR_CONTROLLER_MODE control code
typedef enum {
	Data_Mode,
	Fax_Mode,
	Voice_Mode
} eControllerMode;

// See DevMgr_ConfigureStreamObject function
typedef enum {
	DEVMGR_INIT_STREAM,			// for HDLC stream a CRC (dwParam = 16) must be applied
	DEVMGR_CONNECT_STREAM
} eStreamConfigCode;

// See DevMgr_PMControl function
typedef enum {
	DEVMGR_SetACPIState,
	DEVMGR_EnableRingWake,
	DEVMGR_DisablePMESTS,
	DEVMGR_PMESupportMask,
	DEVMGR_EnableD3CID
} ePmeFunction;

// See DevMgr_PMControl function
typedef enum {
	DEVMGR_D0,
	DEVMGR_D1,
	DEVMGR_D2,
	DEVMGR_D3
} ePmeState;

// See DevMgr_SpeakerControl function
typedef enum {
	DEVMGR_SET_SPEAKER_INPUT,
	DEVMGR_DISABLE_SPEAKER,
	DEVMGR_ENABLE_SPEAKER,
	DEVMGR_DISCONNECT_SPEAKER_FROM_DP,
	DEVMGR_CONNECT_SPEAKER_TO_DP
} eSpeakerControlType;

// See DevMgr_MaskDataTransfer & DevMgr_PurgeData functions
typedef enum {
	DEVMGR_DIR_RX = 0x1,
	DEVMGR_DIR_TX = 0x2,
	DEVMGR_DIR_ALL = 0x3
} eDirection;

// See DevMgr_BeginCtrlNotification function
typedef enum {
	DEVMGR_POLARITY_OFF,
	DEVMGR_POLARITY_ON,
	DEVMGR_POLARITY_ANY
} ePolarity;


// See DevMgr_SetupCallingTone function
typedef enum {
	DEVMGR_TONE_V25CALL,
	DEVMGR_TONE_CNG,
	DEVMGR_TONE_CED,
	DEVMGR_TONE_CI,
} eCALLING_TONE;

// See DevMgr_SetupCallingTone function
// TONEGEN - contains parameters for tone generator.
typedef struct tagTONEGEN {
	UINT32		OnTime;
	UINT32		OffTime;
} TONEGEN, *PTONEGEN;

// The following struct include the parameters for DTMF generator.
typedef struct tagDTMFParamsOut {
	UINT32			Duration;			// millisec
	UINT32			InterDigitDelay;	// millisec
	long			LowBandPower;		// dBm
	long			HighBandPower;		// dBm
} DTMF_PARAMS_OUT, *PDTMF_PARAMS_OUT;

// See DevMgr_SetDialMode function
typedef enum {
	DEVMGR_DIAL_TONE,
	DEVMGR_DIAL_PULSE
} eDialMode;

// See DevMgr_DialDigitAsync function
typedef enum { 
	DIGIT_0,
	DIGIT_1,
	DIGIT_2,
	DIGIT_3,
	DIGIT_4,
	DIGIT_5,
	DIGIT_6,
	DIGIT_7,
	DIGIT_8,
	DIGIT_9,
	DIGIT_STAR,
	DIGIT_POUND,
	DIGIT_A,
	DIGIT_B,
	DIGIT_C,
	DIGIT_D
} eDialDigit;

// see DevMgr_V8bisStartTone
typedef enum {
	MRd_TONE_INIT =		0x21,	// Mode Request-d (Initiating)
	CRe_TONE = 			0x22,	// Initiating 1375+2002+400, Capabilities Request-e (Initiating)
	CRd_TONE_INIT =		0x23,	// Capabilities Request (Initiating)
	CRd_TONE = 			0x33,	// Responding 1529+2225+1900, Capabilities Request-d (Responding)
	VALID_SEG1_TONE =	0xE0
} eV8BIS_TONE_ID;

typedef struct tagMODULATIOM_PARAMS {
	MODULATION_TYPE	ModType;
	BOOL			bAutoRate;
	UINT32			dwMinTxSpeed;
	UINT32			dwMaxTxSpeed;
	UINT32			dwMinRxSpeed;
	UINT32			dwMaxRxSpeed;
	BOOL			buLaw;
	BOOL			bV8bisK56Neg;
	BOOL			bAllowV8;
} MODULATIOM_PARAMS, *PMODULATIOM_PARAMS;

// See V.8 API
typedef struct tagV8Config {
	// bit 0 - 
	// bit 1 - No Tx of ANSam until CI detected
	// bit 2 - Transmit CI
	// bit 3 - 
	// bit 4 - Send GSTN Octet
	// bit 5, 6, 7 - Call Function to be Transmitted
	UINT8	cV8CtrlMask0;
	// bit 0 - Send Protocol Octet
	// bit 1 - Send Minimum JM Sequence
	// bit 2 - Cellular Access
	// bit 3 - No RLSD in V.8
	// bit 4 - 
	// bit 5, 6, 7 - Protocol Bits
	UINT8	cV8CtrlMask1;
} V8Config;

typedef struct tagV8Status {
	// bit 0 - Min CI Sent
	// bit 1 - CI Detected
	// bit 2 - CM Detected
	// bit 3 - CJ Detected
	// bit 4 - JM Detected
	// bit 5 - ANS Detected
	// bit 6 - FSK Byte Ready
	// bit 7 - 
	UINT8	cV8StatMask0;
	// bit 0 - No Modes Common
	// bit 1 - 
	// bit 2 - 
	// bit 3 - ANSam Detected
	// bit 4 - Protocol Octet Received
	// bit 5, 6, 7 - Received Call Function
	UINT8	cV8StatMask1;
	// bit 0 - Received GSTN Octet
	// bit 1 - Remote is using Cellular
	// bit 2 - 
	// bit 3 - 
	// bit 4 - 
	// bit 5, 6, 7 - Received Protocol Bits
	UINT8	cV8StatMask2;
} V8Status;

// See DevMgr_BeginRetrain function
typedef enum {
	DEVMGR_LMON_RETRAIN,
	DEVMGR_LMON_FAST_RETRAIN,
	DEVMGR_LMON_RATE_RENEG,
	DEVMGR_LMON_CLEARDOWN
} eRetrainType;

typedef enum {
	LMON_INACTIVE, 
	LMON_ACTIVE, 
	LMON_RETRAINING, 
	LMON_RRUPSTATE, 
	LMON_RRDOWNSTATE,
	LMON_REMOTERETRAINING, 
	LMON_REMOTERENEGOTIATING, 
	LMON_CARRIERLOST
} LMON_STATE;

typedef UINT32		(__stdcall *DEVMGRGetInterfaceVersion)(void);
typedef HANDLE		(__stdcall *DEVMGRCreate)(PSYSTEM_INSTANCES_T);
typedef COM_STATUS	(__stdcall *DEVMGRDestroy)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGROpen)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRClose)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRConfigure)(HANDLE, eConfigureCode, void*);
typedef COM_STATUS	(__stdcall *DEVMGRMonitor)(HANDLE, eMonitorCode, void*, UINT32);
typedef COM_STATUS	(__stdcall *DEVMGRControl)(HANDLE, eControlCode, void*);

typedef HANDLE		(__stdcall *DEVMGRCreateStreamObject)(HANDLE, eStreamType);
typedef COM_STATUS	(__stdcall *DEVMGRDestroyStreamObject)(HANDLE, HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRConfigureStreamObject)(HANDLE, HANDLE, eStreamConfigCode, void*);

typedef COM_STATUS	(__stdcall *DEVMGRPMControl)(HANDLE, ePmeFunction, ePmeState);
typedef COM_STATUS	(__stdcall *DEVMGRSpeakerControl)(HANDLE, eSpeakerControlType, UINT8);
typedef COM_STATUS	(__stdcall *DEVMGRMaskDataTransfer)(HANDLE, eDirection, BOOL);
typedef COM_STATUS	(__stdcall *DEVMGRPurgeData)(HANDLE, eDirection);
typedef COM_STATUS	(__stdcall *DEVMGRSetRingDetect)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRCancelRingDetect)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRBeginCtrlNotification)(HANDLE, HANDLE, ePolarity);
typedef COM_STATUS	(__stdcall *DEVMGREndCtrlNotification)(HANDLE, HANDLE);
typedef HANDLE		(__stdcall *DEVMGRCreateToneDetector)(HANDLE, eTONEDETECT_TYPE);
typedef COM_STATUS	(__stdcall *DEVMGRStartToneDetector)(HANDLE, HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRStopToneDetector)(HANDLE, HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRDeleteToneDetector)(HANDLE, HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRSetupCallingTone)(HANDLE, eCALLING_TONE, PTONEGEN);
typedef COM_STATUS	(__stdcall *DEVMGRStartCallingTone)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRStopCallingTone)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRSetupDTMFGenerator)(HANDLE, PDTMF_PARAMS_OUT);
typedef COM_STATUS	(__stdcall *DEVMGRSetDialMode)(HANDLE, eDialMode, UINT8);
typedef COM_STATUS	(__stdcall *DEVMGRDialDigitAsync)(HANDLE, eDialDigit);
typedef COM_STATUS	(__stdcall *DEVMGRCancelDialMode)(HANDLE);

typedef COM_STATUS	(__stdcall *DEVMGRV8bisSet)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisStartToneDetector)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisStopToneDetector)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisStartTone)(HANDLE, eV8BIS_TONE_ID);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisStopTone)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisSetupV21)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisSetupMode)(HANDLE, BOOL);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisSendMsg)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisTxOff)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRV8bisCancel)(HANDLE, BOOL);

typedef COM_STATUS	(__stdcall *DEVMGRConfigureMsParams)(HANDLE, MODULATIOM_PARAMS*);
typedef COM_STATUS	(__stdcall *DEVMGRV8SetConfig)(HANDLE, V8Config*, BOOL);
typedef COM_STATUS	(__stdcall *DEVMGRV8GetConfig)(HANDLE, V8Config*);
typedef COM_STATUS	(__stdcall *DEVMGRV8GetStatus)(HANDLE, V8Status*);
typedef COM_STATUS	(__stdcall *DEVMGRActivateMsParams)(HANDLE);

typedef COM_STATUS	(__stdcall *DEVMGRUpdateRateMask)(HANDLE);
typedef COM_STATUS	(__stdcall *DEVMGRSetAutoRateControl)(HANDLE, UINT8);
typedef COM_STATUS	(__stdcall *DEVMGRBeginRetrain)(HANDLE, UINT32, eRetrainType);
typedef COM_STATUS	(__stdcall *DEVMGRClearRemoteRetrainDetect)(HANDLE);

typedef struct tagI_DEVICE_MGR_T {
	DEVMGRGetInterfaceVersion			GetInterfaceVersion;		// +
	DEVMGRCreate						Create;						// +
	DEVMGRDestroy						Destroy;					// +
	DEVMGROpen							Open;						// +
	DEVMGRClose							Close;						// +
	DEVMGRConfigure						Configure;					// +
	DEVMGRMonitor						Monitor;					// +
	DEVMGRControl						Control;					// +
	DEVMGRCreateStreamObject			CreateStreamObject;			// +
	DEVMGRDestroyStreamObject			DestroyStreamObject;		// +
	DEVMGRConfigureStreamObject			ConfigureStreamObject;		// +
	DEVMGRPMControl						PMControl;					// -
	DEVMGRSpeakerControl				SpeakerControl;				// +
	DEVMGRMaskDataTransfer				MaskDataTransfer;			// +
	DEVMGRPurgeData						PurgeData;					// +
	DEVMGRSetRingDetect					SetRingDetect;				// +
	DEVMGRCancelRingDetect				CancelRingDetect;			// +
	DEVMGRBeginCtrlNotification			BeginCtrlNotification;		// +
	DEVMGREndCtrlNotification			EndCtrlNotification;		// +
	DEVMGRCreateToneDetector			CreateToneDetector;			// +
	DEVMGRStartToneDetector				StartToneDetector;			// +
	DEVMGRStopToneDetector				StopToneDetector;			// +
	DEVMGRDeleteToneDetector			DeleteToneDetector;			// +
	DEVMGRSetupCallingTone				SetupCallingTone;			// +
	DEVMGRStartCallingTone				StartCallingTone;			// +
	DEVMGRStopCallingTone				StopCallingTone;			// +
	DEVMGRSetupDTMFGenerator			SetupDTMFGenerator;			// +
	DEVMGRSetDialMode					SetDialMode;				// +
	DEVMGRDialDigitAsync				DialDigitAsync;				// +
	DEVMGRCancelDialMode				CancelDialMode;				// +
	DEVMGRV8bisSet						V8bisSet;					// +
	DEVMGRV8bisStartToneDetector		V8bisStartToneDetector;		// +
	DEVMGRV8bisStopToneDetector			V8bisStopToneDetector;		// +
	DEVMGRV8bisStartTone				V8bisStartTone;				// +
	DEVMGRV8bisStopTone					V8bisStopTone;				// +
	DEVMGRV8bisSetupV21					V8bisSetupV21;				// +
	DEVMGRV8bisSetupMode				V8bisSetupMode;				// +
	DEVMGRV8bisSendMsg					V8bisSendMsg;				// +
	DEVMGRV8bisTxOff					V8bisTxOff;					// +
	DEVMGRV8bisCancel					V8bisCancel;				// +
	DEVMGRConfigureMsParams				ConfigureMsParams;			// +
	DEVMGRV8SetConfig					V8SetConfig;				// +
	DEVMGRV8GetConfig					V8GetConfig;				// +
	DEVMGRV8GetStatus					V8GetStatus;				// +
	DEVMGRActivateMsParams				ActivateMsParams;			// +
	DEVMGRUpdateRateMask				UpdateRateMask;				// +
	DEVMGRSetAutoRateControl			SetAutoRateControl;			// +
	DEVMGRBeginRetrain					BeginRetrain;				// +
	DEVMGRClearRemoteRetrainDetect		ClearRemoteRetrainDetect;	// +
} I_DEVICE_MGR_T, *PI_DEVICE_MGR_T;
#ifdef __cplusplus
}
#endif

#if ( USE_PRAGMA_PACK == 1 )
#pragma pack(pop)   // Packing compatible with our VXDs
#endif

#endif /* __DRV_MGR_E_H__ */
