/*************************************************************************

File Name: OsString.h

File Description: This file implements all the string/memory utilities.

Function list:

GLOBAL:

STATIC:


************************************************************************/

#ifndef __OS_STRING_EX_H__
#define __OS_STRING_EX_H__

#ifdef __cplusplus
extern "C"
{
#endif

#if ( USE_PRAGMA_PACK == 1 )
#pragma pack(push,1)   // Packing compatible with WinAC
#endif

/********************************************************************************

Function Name: OsStrCpy.

Function Description:The function copies a specified a string from one 
					 buffer to another. This function may not work correctly if 
					 the buffers overlap. 

Arguments: BuffDest - Pointer to buffer to contain copied string.
		   BuffSrc - Pointer to buffer with the string to copy.

Return Value: A pointer to BuffDest.

********************************************************************************/
PVOID	OsStrCpy(OUT PCHAR BuffDest, IN PCHAR BuffSrc);

/********************************************************************************

Function Name: OsStrnCpy.

Function Description:The function copies a specified a string from one 
					 buffer to another. The function does not copy more 
					 than MaxSize bytes.The behavior of strncpy is 
					 undefined if the source and destination strings overlap

Arguments: BuffDest - Pointer to buffer to contain copied string.
		   BuffSrc - Pointer to buffer with the string to copy.
		   MaxSize - Max chars to copy

Return Value: A pointer to BuffDest.

********************************************************************************/
PVOID	OsStrnCpy(OUT PCHAR BuffDest, IN PCHAR BuffSrc, int MaxSize);

/********************************************************************************

Function Name: OsStrCat.

Function Description:The function copies a specified a string from one 
					 buffer to another, at its end. This function may not work correctly if 
					 the buffers overlap. 

Arguments: BuffDest - Pointer to buffer to contain copied string.
		   BuffSrc - Pointer to buffer with the string to copy.

Return Value: A pointer to BuffDest.

********************************************************************************/
PVOID	OsStrCat(OUT PCHAR BuffDest, IN PCHAR BuffSrc);

/********************************************************************************

Function Name: OsStrnCat.

Function Description:The function copies a specified a string from one 
					 buffer to another, at its end. The function does not copy
					 more than MaxSize bytes.The behavior of strncpy is 
					 undefined if the source and destination strings overlap

Arguments: BuffDest - Pointer to buffer to contain copied string.
		   BuffSrc - Pointer to buffer with the string to copy.
		   MaxSize - Max chars to copy

Return Value: A pointer to BuffDest.

********************************************************************************/
PVOID	OsStrnCat(OUT PCHAR BuffDest, IN PCHAR BuffSrc, int MaxSize);

/********************************************************************************

Function Name: OsStrCmp.

Function Description:The strcmp function compares string1 and string2 
		lexicographically and returns a value indicating their relationship

Arguments: string1 ,string2 - Null terminated strings to compare

Return Value: lexicographic relation of string1 to string2.

********************************************************************************/
int	OsStrCmp(IN PCHAR string1, IN PCHAR string2);
/********************************************************************************

Function Name: OsStrnCpy.

Function Description:The strcmp function compares string1 and string2 
		lexicographically and returns a value indicating their relationship
Arguments: string1 ,string2 - Null terminated strings to compare
					MaxSize	- Number of characters to compare

Return Value: lexicographic relation of string1 to string2.

********************************************************************************/
int	OsStrnCmp(IN PCHAR string1, IN PCHAR string2, int MaxSize);
/********************************************************************************

Function Name: OsToupper.

Function Description:converts a charecter c to upper case  

Arguments: c - charected to be converted to upper case.

Return Value: upper case charecter of c.

********************************************************************************/
int	OsToupper(int c);
/********************************************************************************

Function Name: OsTolower.

Function Description:converts a charecter c to lower case  

Arguments: c - charected to be converted to lower case.

Return Value: lower case charecter of c.

********************************************************************************/
int	OsTolower(int c);
/********************************************************************************

Function Name: OsIsDigit.

Function tests if a charecter is a decimal digit. 

Arguments: c - integer to test.

Return Value: non zero value if c is a decimal digit('0' - '9').

********************************************************************************/
int	OsIsDigit(int c);

/********************************************************************************

Function Name: OsSprintf.

Function Description:The OsSprintf function formats and stores a series of 
						characters and values in buffer. 

Arguments: buffer - Pointer to buffer to contain copied string.
		   format - control string


Return Value: OsSprintf returns the number of bytes stored in buffer, 
				not counting the terminating null character. 

********************************************************************************/
int	OsSprintf(OUT PCHAR buffer, PCHAR format_string, ...);

/********************************************************************************

Function Name: OSSprintfFixedArgs.

Function Description:The OSSprintfFixedArgs function formats and stores a series of 
						characters and values in buffer. The function recieves the 
						format parameters not on the stack, but using the va_list
						parameter

Arguments: buffer - Pointer to buffer to contain copied string.
		   format - control string.
		   StartedVaList - pointer to va_list that has been started to include the format arguments


Return Value: OsSprintf returns the number of bytes stored in buffer, 
				not counting the terminating null character. 

********************************************************************************/
int OSSprintfFixedArgs(PCHAR buffer, PCHAR format, PCHAR StartedVaList);

/********************************************************************************

Function Name: OsStrLen.

Function Description:The function returns the length of a null-terminated string.

Arguments: String - Pointer to buffer to contain copied string.

Return Value: int - the length of a null-terminated string.

********************************************************************************/
int	OsStrLen(IN PCHAR String);

 
int Osatoi(IN PCHAR String);
 
#if ( USE_PRAGMA_PACK == 1 )
#pragma pack(pop)   // Packing compatible with our VXDs
#endif

#ifdef __cplusplus
}
#endif

#endif /* __OS_STRING_EX_H__ */
