# makefile for octopus module under linux

include $(MAK_ENV)

OCTOPUS_ODIR = $(OCTOPUS_DIR)/obj
OCTOPUS_MODULE = $(OCTOPUS_ODIR)/mod_octopus.o
ALL_MODULES += $(OCTOPUS_MODULE)

ifneq ($(MAIN_MAKE),yes)

INCLUDES =	-I $(COMMON_DIR)/include		\
			-I $(OCTOPUS_DIR)/include		\
			-I $(OCTOPUS_DIR)/osspecific	\
			-I $(SW_MODEM_DIR)/include		\
			-I $(SW_MODEM_DIR)/dspmodls		\
			-I $(SW_MODEM_DIR)/linux		\
			-I $(SW_MODEM_DIR)/sw_modules	\
			-I $(INF2BIN_DIR)				\
			-I $(SES_CTRL_DIR)/include		\
			-I $(DEVMGR_DIR)/include	

C_FLAGS =	$(INCLUDES)		\
			$(ALL_FLAGS)

all : ALLODIRS octopus_module


OCT_CFILES  = 
OCT_CFILES += $(OCTOPUS_DIR)/configurationmgr/configmgr.c
OCT_CFILES += $(OCTOPUS_DIR)/debugmgr/dbgmgr.c
OCT_CFILES += $(OCTOPUS_DIR)/memorymgr/memmgr.c
OCT_CFILES += $(OCTOPUS_DIR)/nvmmgr/nvmmgr.c
OCT_CFILES += $(OCTOPUS_DIR)/runtimemgr/rtmgr.c
OCT_CFILES += $(OCTOPUS_DIR)/statusmgr/statmgr.c

OCTC_ODIR = $(OCTOPUS_ODIR)/oct_cfiles
ALL_ODIRS += $(OCTC_ODIR)

OCTC_OBJS = $(patsubst %.c,$(OCTC_ODIR)/%.o,$(notdir $(OCT_CFILES)))

vpath %.c $(OCTOPUS_DIR)/osspecific $(OCTOPUS_DIR)/configurationmgr $(OCTOPUS_DIR)/debugmgr $(OCTOPUS_DIR)/memorymgr $(OCTOPUS_DIR)/interfacecontroller $(OCTOPUS_DIR)/nvmmgr $(OCTOPUS_DIR)/runtimemgr $(OCTOPUS_DIR)/statusmgr

$(OCTC_ODIR)/%.o: %.c
	cc $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GCC_TEMPS_EXT),$(notdir $@)) $(OCTC_ODIR)

OS_SPECIFIC_FILES  = 

OS_SPECIFIC_ODIR = $(OCTOPUS_ODIR)/os_specific_files
ALL_ODIRS += $(OS_SPECIFIC_ODIR)

OS_SPECIFIC_OBJS = $(patsubst %.c,$(OS_SPECIFIC_ODIR)/%.o,$(notdir $(OS_SPECIFIC_FILES)))

vpath %.c $(OCTOPUS_DIR)/osspecific

$(OS_SPECIFIC_ODIR)/%.o: %.c
	cc $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GCC_TEMPS_EXT),$(notdir $@)) $(OS_SPECIFIC_ODIR)

UTILITIES_FILES  = 
UTILITIES_FILES += $(OCTOPUS_DIR)/utilities/hashtable.c
UTILITIES_FILES += $(OCTOPUS_DIR)/utilities/linkedlist.c
UTILITIES_FILES += $(OCTOPUS_DIR)/utilities/oct_globals.c
UTILITIES_FILES += $(OCTOPUS_DIR)/utilities/crc32.c
UTILITIES_FILES += $(OCTOPUS_DIR)/linux/octwrap.c

UTILITIES_ODIR = $(OCTOPUS_ODIR)/utilities
ALL_ODIRS += $(UTILITIES_ODIR)

UTILITIES_OBJS = $(patsubst %.c,$(UTILITIES_ODIR)/%.o,$(notdir $(UTILITIES_FILES)))

vpath %.c $(OCTOPUS_DIR)/utilities $(OCTOPUS_DIR)/linux

$(UTILITIES_ODIR)/%.o: %.c
	cc $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GCC_TEMPS_EXT),$(notdir $@)) $(UTILITIES_ODIR)

ALL_LIBS =	$(OCTC_OBJS)		\
			$(OS_SPECIFIC_OBJS)	\
			$(UTILITIES_OBJS)

$(OCTOPUS_MODULE): $(ALL_LIBS)
	$(LD) -o $@ $(LD_FLAGS) -r $^

octopus_module: $(OCTOPUS_MODULE)

# Output directories rule (must be after all ALL_ODIRS += statements:
ALLODIRS: $(ALL_ODIRS)

$(ALL_ODIRS):
	mkdir -p $@

endif # of if (MAIN_MAKE)

# DO NOT DELETE
