#include "ossysenv.h"
#include "devcomm_ex.h"
#include "devcomm_in.h"
#include "vxdcomm.h"
#include "memmgr_ex.h"
#include "rtmgr_ex.h"
#include "statmgr_ex.h"
#include "configmgr_ex.h"
#include "nvmmgr_ex.h"
#include "dbgmgr_ex.h"
 
#if ( ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER)) || \
      ( OS_TYPE == OS_LINUX ) )
#if ( PROJECT == PRJ_HSF)
 
#include "devmgr_ex.h"
#include "comctrl_ex.h"
#include "atparser_ex.h"
 
void *GetBasic2Funcs(void);
void *GetIchFuncs(void);
void *SessionMgrGetInterface(void);
void *GetTonesInterface(void);
void *GetV34Interface(void);
void *GetFsksInterface(void);
void *GetV90Interface(void);
void *GetFallBackInterface(void);

#endif /* PRJ_HSF */
#endif /* DLL or LINUX */
 

#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
HANDLE	LoadVxd( PCHAR FileName );
void	UnloadVxd( PCHAR FileName );
PVOID	GetFunctionTable(	PCHAR		FileName , 
							BLOCK_TYPE	BlockType, 
							UINT32		BlockID);

BLOCK_TO_DEVICE_MAP_T BlockToDeviceMap[BLOCK_NUM] =
{
	/* BlockType */		/* BlockID */	/* DeviceIndex */
	{	BLOCK_MEM_MGR,		0,				MEM_MGR_DEV_INDEX},
	{	BLOCK_RT_MGR,		0,				RT_MGR_DEV_INDEX},
	{	BLOCK_STATUS_MGR,	0,				STATUS_MGR_DEV_INDEX},
	{	BLOCK_CONFIG_MGR,	0,				CONFIG_MGR_DEV_INDEX},
	{	BLOCK_DEBUG_MGR,	0,				DEBUG_MGR_DEV_INDEX},
	{	BLOCK_NVM_MGR,		0,				NVM_MGR_DEV_INDEX},
	{	BLOCK_SCR_SRV,		0,				SCR_SRV_DEV_INDEX},
	
};

DEVICE_INFO_T	DeviceInfoTable[DEVICE_NUM] =
{
	/* DeviceName */	/* nClients */
	{"Mem_Mgr.vxd",			0},
	{"RTMgrDrv.vxd",		0},
	{"Stat_Mgr.vxd",		0},
	{"Cnfg_Mgr.vxd",		0},
	{"Dbg_Mgr.vxd",			0},
	{"NVM_Mgr.vxd",			0},
	{"ScrSrv.vxd",			0},
};

HANDLE	ConnectBlock(			BLOCK_TYPE	BlockType , 
								UINT32		BlockID )
{
	int i;
	for ( i = 0; i < BLOCK_NUM ; i++ )
	{
		if (	(BlockToDeviceMap[i].BlockType	==	BlockType ) &&
				(BlockToDeviceMap[i].BlockID	==	BlockID)	)
		{
			int DeviceIndex = BlockToDeviceMap[i].DeviceIndex;
			if ( 0 == DeviceInfoTable[DeviceIndex].nClients )
			{
				LoadVxd(DeviceInfoTable[DeviceIndex].DeviceName);
			}
			DeviceInfoTable[DeviceIndex].nClients ++;
			return (HANDLE)&BlockToDeviceMap[i];
		}
	}
	return 0;
}

VOID	DisconnectBlock(		HANDLE		hBlock	)
{
	PBLOCK_TO_DEVICE_MAP_T	pBlockToDevMap;
	PDEVICE_INFO_T pDeviceInfo;
	if ( 0 == hBlock )
		return;
	pBlockToDevMap = (PBLOCK_TO_DEVICE_MAP_T)hBlock;
	pDeviceInfo = &DeviceInfoTable[pBlockToDevMap->DeviceIndex];
	if ( 0 ==  --(pDeviceInfo -> nClients) )
	{
		UnloadVxd(pDeviceInfo->DeviceName);
	}
}

PVOID	GetInterfaceFuncTbl(	HANDLE		hBlock	)
{
	PBLOCK_TO_DEVICE_MAP_T	pBlockToDevMap;
	PDEVICE_INFO_T pDeviceInfo;
	if ( 0 == hBlock  )
		return NULL;
	pBlockToDevMap = (PBLOCK_TO_DEVICE_MAP_T)hBlock;
	pDeviceInfo = &DeviceInfoTable[pBlockToDevMap->DeviceIndex];
	return GetFunctionTable(
		pDeviceInfo->DeviceName, 
		pBlockToDevMap->BlockType, 
		pBlockToDevMap->BlockID);
}




char ErrorMsg[50];
char TOUPPER(char c)
{
	if ( (c >= 'a') && (c <= 'z'))
		return (char)(c + 'A' - 'a');
	return c;
}
void	ConvertFileNameToLogicalName( PCHAR FileName, PCHAR LogicalName )
{
	int i;
	for ( i = 0 ; (i < 8) && ( i < (int)strlen(FileName) ) ; i++)
	{
		if ( FileName[i] == ' ' || FileName[i] == '.' )
		{
			LogicalName[i] = '\0';
			break;
		} else {
			LogicalName[i] = TOUPPER( FileName[i] );
		}
	}

	LogicalName[8] = '\0';
}

void	ConvertFileNameTo8charName( PCHAR FileName, PCHAR LogicalName )
{
	int i;
	for ( i = 0 ; (i < 8) && ( i < (int)strlen(FileName) ) ; i++)
	{
		if ( FileName[i] == '.' )
			break;
		LogicalName[i] = TOUPPER( FileName[i] );
	}
	for ( ; i < 8 ; i++ ) {
		LogicalName[i] = ' ';
	}

	LogicalName[8] = '\0';
}

HANDLE	LoadVxd( PCHAR FileName )
{
	PDDB		pDDB;
	PDEVICEINFO pDeviceInfo;
	PCHAR		Msg = ErrorMsg;
	char		Name8char[10];
	UINT32		rc;


	ConvertFileNameTo8charName( FileName, Name8char );
	pDDB = Get_DDB( 0, Name8char );
	if ( pDDB != NULL ) {
		return (HANDLE)pDDB;
	}
	rc = VXDLDR_LoadDevice(FileName, 
							     VXDLDR_INIT_DEVICE, 
								 &pDeviceInfo,
								 &pDDB);
	if ( rc != 0 ) 
	{
		if ( rc == VXDLDR_ERR_BAD_DEVICE_FILE )
			sprintf( Msg, "= = = ERROR: %s Specified VxD file is not usable.", 
									FileName );
		else if ( rc == VXDLDR_ERR_DEVICE_REFUSED )
			sprintf( Msg, "= = = ERROR: %s Specified device operation not allowed.", 
									FileName );
		else if ( rc == VXDLDR_ERR_DUPLICATE_DEVICE )
			sprintf( Msg, "= = = ERROR: %s Specified operation has already occurred for the given device.",
									FileName );
		else if ( rc == VXDLDR_ERR_FILE_OPEN_ERROR )
			sprintf( Msg, "= = = ERROR: %s Unable to open specified file.", 
									FileName );
		else if ( rc == VXDLDR_ERR_FILE_READ )
			sprintf( Msg, "= = = ERROR: %s Unable to read specified file.", 
									FileName );
		else if ( rc == VXDLDR_ERR_IN_DOS )
			sprintf( Msg, "= = = ERROR: %s System error.", 
									FileName );
		else if ( rc == VXDLDR_ERR_NO_SUCH_DEVICE )
			sprintf( Msg, "= = = ERROR: %s Specified device not found.", 
									FileName );
		else if ( rc == VXDLDR_ERR_OUT_OF_MEMORY )
			sprintf( Msg, "= = = ERROR: %s Out of memory.", 
									FileName );
		else if ( rc == VXDLDR_ERR_DEVICE_UNLOADABLE )
			sprintf( Msg, "= = = ERROR: %s Device Unloadable", 
									FileName );
		else 
			sprintf( Msg, "= = = ERROR: %s Undocumented error %d", 
									FileName, rc );

		return NULL;
	}
	return (HANDLE)pDDB;
}

void	UnloadVxd( PCHAR FileName )
{
	PCHAR		Msg = ErrorMsg;
	UINT32		rc;
	char		LogicalName[10];

	ConvertFileNameToLogicalName( FileName, LogicalName );

	rc = VXDLDR_UnloadDevice(-1, LogicalName);

	if ( rc != 0 ) {
		if ( rc == VXDLDR_ERR_BAD_DEVICE_FILE )
			sprintf( Msg, "%s", "= = = ERROR:: Specified VxD file is not usable." );
		else if ( rc == VXDLDR_ERR_DEVICE_REFUSED )
			sprintf( Msg, "%s", "= = = ERROR:: Specified device operation not allowed." );
		else if ( rc == VXDLDR_ERR_DUPLICATE_DEVICE )
			sprintf( Msg, "%s", "= = = ERROR:: Specified operation has already occurred for the given device." );
		else if ( rc == VXDLDR_ERR_FILE_OPEN_ERROR )
			sprintf( Msg, "%s", "= = = ERROR:: Unable to open specified file." );
		else if ( rc == VXDLDR_ERR_FILE_READ )
			sprintf( Msg, "%s", "= = = ERROR:: Unable to read specified file." );
		else if ( rc == VXDLDR_ERR_IN_DOS )
			sprintf( Msg, "%s", "= = = ERROR:: System error." );
		else if ( rc == VXDLDR_ERR_NO_SUCH_DEVICE )
			sprintf( Msg, "%s", "= = = ERROR:: Specified device not found." );
		else if ( rc == VXDLDR_ERR_OUT_OF_MEMORY )
			sprintf( Msg, "%s", "= = = ERROR:: Out of memory." );
		else if ( rc == VXDLDR_ERR_DEVICE_UNLOADABLE )
			sprintf( Msg, "%s", "= = = ERROR:: Device Unloadable" );
		else 
			sprintf( Msg, "= = = ERROR:: Undocumented error %d", rc );

	}
}

PVOID	GetFunctionTable( PCHAR FileName , BLOCK_TYPE BlockType, UINT32 BlockID)
{
	PDDB		Device_pDBB;
	ALLREGS		AllRegs;
	UINT32		My_Eax;
	char		Device8charName[10];
	ConvertFileNameTo8charName(FileName, Device8charName);
	Device_pDBB = Get_DDB( 0, Device8charName );
	if (Device_pDBB == NULL) {
		sprintf( ErrorMsg, "CDeviceLoader : Driver not loaded %s", Device8charName );
		return NULL;
	}


	memset( &AllRegs, 0, sizeof(ALLREGS) );
	AllRegs.REDX = BlockType;
	AllRegs.RESI = (UINT32)BlockID;
	Directed_Sys_Control(Device_pDBB, GET_ENTRY_POINT, &AllRegs);	
	_asm mov My_Eax, eax  // return value from the "VOID" function...

	ErrorMsg[0] = '\0';

	return (PVOID)My_Eax;	
}

#endif // OS_VXD

#if ( ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER)) || \
      ( OS_TYPE == OS_LINUX ) )
#if ( PROJECT == PRJ_HSF)
BLOCK_TO_INTERFACE_MAP_T BlockToInterfaceMap[] =
{
	/* BlockType */		/* BlockID */	/* GetInterface */
	{	BLOCK_MEM_MGR,		0,				MemMgrGetInterface		},
	{	BLOCK_RT_MGR,		0,				RTMgrGetInterface		},
	{	BLOCK_STATUS_MGR,	0,				StatMgrGetInterface		},
	{	BLOCK_CONFIG_MGR,	0,				ConfigMgrGetInterface	},
	{	BLOCK_DEBUG_MGR,	0,				DebugMgrGetInterface	},
	{	BLOCK_NVM_MGR,		0,				NVMMgrGetInterface		},
	{	BLOCK_SCR_SRV,		0,				NULL					},
	{	BLOCK_HW_NONE,		0,				NULL					},
	{	BLOCK_HW_BASIC2,	0,				GetBasic2Funcs			},
	{	BLOCK_HW_ICH,		0,				GetIchFuncs  			},
	{	BLOCK_DEVICE_MGR,	0,				DevMgrGetInterface		},
    {	BLOCK_COM_CTRL,		0,				ComCtrlGetInterface     },
    {	BLOCK_MEM_MGR,		0,				MemMgrGetInterface      },
    {	BLOCK_RT_MGR,		0,				RTMgrGetInterface       },
    {	BLOCK_AT_DTE,		0,				AtDteGetInterface       },
	{	BLOCK_TONE_DET,		0,				GetTonesInterface		},
	{	BLOCK_TONE_GEN,		0,				GetTonesInterface		},
	{	BLOCK_ANS_DET,		0,				GetTonesInterface		},
	{	BLOCK_ANS_GEN,		0,				GetTonesInterface		},
	{	BLOCK_DTMF_DET,		0,				GetTonesInterface		},
	{	BLOCK_DTMF_GEN,		0,				GetTonesInterface		},
	{	BLOCK_CR_DET,		0,				GetTonesInterface		},
	{	BLOCK_CR_GEN,		0,				GetTonesInterface		},
	{	BLOCK_SESSION_MGR,	0,				SessionMgrGetInterface	},
	{	BLOCK_V90,			0,				GetV90Interface			},
	{	BLOCK_V34,			0,				GetV34Interface			},
	{	BLOCK_V8,			0,				GetFsksInterface		},
	{	BLOCK_V21,			0,				GetFsksInterface		},
	{	BLOCK_V23,			0,				GetFsksInterface		},
	{	BLOCK_BELL103,		0,				GetFsksInterface        },
	{	BLOCK_V32BIS,		0,				GetFallBackInterface	},
	{	BLOCK_V32,		    0,				GetFallBackInterface	},
	{	BLOCK_V22BIS,		0,				GetFallBackInterface	},
	{	BLOCK_V22,		    0,				GetFallBackInterface	},
	{	BLOCK_BELL212,		0,				GetFallBackInterface    },
};
#endif /* PROJECT == PRJ_HSF */
#if ( PROJECT == PRJ_SDSL )
BLOCK_TO_INTERFACE_MAP_T BlockToInterfaceMap[BLOCK_NUM] =
{
	/* BlockType */		/* BlockID */	/* GetInterface */
	{	BLOCK_MEM_MGR,		0,				MemMgrGetInterface},
	{	BLOCK_RT_MGR,		0,				RTMgrGetInterface},
	{	BLOCK_STATUS_MGR,	0,				StatMgrGetInterface},
	{	BLOCK_CONFIG_MGR,	0,				ConfigMgrGetInterface},
	{	BLOCK_DEBUG_MGR,	0,				DebugMgrGetInterface},
	{	BLOCK_NVM_MGR,		0,				NVMMgrGetInterface},
	{	BLOCK_SCR_SRV,		0,				NULL},
};
#endif /* PROJECT == PRJ_SDSL */

#if ( OS_TYPE == OS_LINUX )

HANDLE	ConnectBlock(			BLOCK_TYPE	BlockType , 
								UINT32		BlockID )
{
	int i;
	for ( i = 0; i < sizeof(BlockToInterfaceMap)/sizeof(BLOCK_TO_INTERFACE_MAP_T); i++ )
	{
		if (	(BlockToInterfaceMap[i].BlockType	==	BlockType ) &&
				(BlockToInterfaceMap[i].BlockID	==	BlockID)	)
		{
			return (HANDLE)i;
		}
	}
	return (HANDLE)-1;
}

VOID	DisconnectBlock(		HANDLE		hBlock	)
{
}

#elif (OS_TYPE == OS_WIN_9X )

HANDLE	ConnectBlock(			BLOCK_TYPE	BlockType , 
								UINT32		BlockID )
{
	int i;
	for ( i = 0; i < BLOCK_NUM ; i++ )
	{
		if (	(BlockToInterfaceMap[i].BlockType	==	BlockType ) &&
				(BlockToInterfaceMap[i].BlockID	==	BlockID)	)
		{
			if ( BlockType == BLOCK_SCR_SRV)
			// ScrSrv is an execption since it is implemented in an
			// externall DLL
			{
				HINSTANCE	hScrSrv;
				GET_ENTRY_POINT_FUNC	FuncGetEntryPoint;
				DWORD					LastError;
				hScrSrv = LoadLibrary("ScrSrv.dll");

				if ( NULL != hScrSrv)
				{
					FuncGetEntryPoint = (GET_ENTRY_POINT_FUNC)GetProcAddress(hScrSrv, "GetEntryPoint");
					if ( NULL != FuncGetEntryPoint)
					{
						BlockToInterfaceMap[i].GetInterface =(FUNC_GET_INTERFACE)FuncGetEntryPoint(BlockType, BlockID);
						return (HANDLE)i;
					}
						
				}
				LastError = GetLastError();
				return (HANDLE)-1;
			}
			return (HANDLE)i;
		}
	}
	return (HANDLE)-1;
}

VOID	DisconnectBlock(		HANDLE		hBlock	)
{
	if ( BlockToInterfaceMap[(int)hBlock].BlockType == BLOCK_SCR_SRV)
	{
		HINSTANCE	hScrSrv;
		hScrSrv = GetModuleHandle("ScrSrv.dll");
		if ( NULL != hScrSrv)
			FreeLibrary(hScrSrv);
	}
}
#endif /* OS_TYPE == OS_WIN_9X */

PVOID	GetInterfaceFuncTbl(	HANDLE		hBlock	)
{
	if ( (HANDLE)-1 == hBlock  )
		return NULL;
	if (NULL == BlockToInterfaceMap[(int)hBlock].GetInterface)
		return NULL;
	return BlockToInterfaceMap[(int)hBlock].GetInterface();
}

#endif //OS_APPLICATION_MODE


