#include <linux/config.h>
#include <linux/pci.h>
#include <linux/sched.h>
#include <linux/malloc.h>
#include <linux/pci.h>
#include <linux/time.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <linux/interrupt.h>

#include "ossysenv.h"
#include "linuxres.h"
#include "cpucnt.inl"


 
struct tq_struct basic2BH;
DEV_DATA devdata;
PISR_FUNC  pISRGlobFunc = NULL;

#define NUM_OF_DEVICE_IDS 4
unsigned int device_id[NUM_OF_DEVICE_IDS] = {0x2013,0x2014,0x2015,0x2016};

//extern void LinuxGlobalISR(void *refdata);
void LinuxSetGlobalISRFunc(PISR_FUNC pFunc)
{
	printk("LinuxSetGlobalISRFunc - pFunc = 0x%x\n",pFunc);
	pISRGlobFunc = pFunc;
}
void	LinuxGetHwPciRes(void **data, unsigned long *pbaseaddr0, unsigned long *pbaseaddr1, unsigned short DeviceId, unsigned short VendorId)
{
	printk("LinuxGetHwPciRes 0x%x 0x%x\n",DeviceId,VendorId );
#ifdef CONFIG_PCI
	if (pcibios_present())
	{
		struct pci_dev *pdev = NULL;

		pdev = pci_find_device(VendorId, DeviceId, pdev);
		if (NULL != pdev)
		{
			*data = pdev;
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )
			*pbaseaddr0 = pdev->base_address[0];
			*pbaseaddr1 = pdev->base_address[1];
#else
			*pbaseaddr0 = pci_resource_start(pdev, 0);
			*pbaseaddr1 = pci_resource_start(pdev, 1);
  			if ( pci_enable_device(pdev) ) 
  			{ 
  				printk("LINUXRES:::: Error - pci_enable_device failed\n"); 
  			} 
 
  			
#endif
 			printk("LINUXRES:::: before pci_set_master\n"); 
 			pci_set_master(pdev); 
			
			// cancel the LSB for io address 
			*pbaseaddr0 = (*pbaseaddr0) & 0xFFFFFFFE;
			*pbaseaddr1 = (*pbaseaddr1) & 0xFFFFFFFE;
 
			devdata.devirq = pdev->irq;
			printk("Found device !!!\n");
			printk("Hardware base addr 0 : 0x%lx\n",*pbaseaddr0);
			printk("Hardware base addr 1 : 0x%lx\n",*pbaseaddr1);
			printk("Hardware interrupt line 0x%x\n",pdev->irq);
		}
	}
	else
	{
		printk("!!!!! pcibios_present FALSE\n");
	}
#endif
}


void	LinuxIntHandler(int irq, void *devid, struct pt_regs *regs)
{
	DEV_DATA *tempdata = (DEV_DATA *)devid;
/*  	static unsigned char count = 0xff; */
/*  	IO_DEB_MESSAGEG(count--); */
/*  	if ( count == 0 ) */
/*  	{ */
/*  		count = 0xff; */
/*  	} */
/*  	printk("LinuxIntHandler 0x%x\n",tempdata->devrefdata); */
//	LinuxGlobalISR( tempdata->devrefdata );
	if(pISRGlobFunc != NULL)
	{
		pISRGlobFunc(tempdata->devrefdata );
	}
}

void	LinuxHookInterrupt(void *data, void (*isr)(void *data),void *refdata)
{
	int result;
	struct pci_dev *dev = (struct pci_dev *)data;
	devdata.devrefdata = refdata;
	result = request_irq(dev->irq, LinuxIntHandler, SA_SHIRQ, "HardwareHandler", &devdata); 
//	printk("<1>Test: LinuxHookInterrupt result =  %d\n",result);
}

void	LinuxUnhookInterrupt(void *data)
{
	struct pci_dev *dev = (struct pci_dev *)data;
	free_irq(dev->irq, &devdata); 
//	printk("<1>Test: LinuxUnhookInterrupt\n");
}

void	*LinuxMapPhys(unsigned long offset, unsigned long size)
{
	return ioremap(offset, size);
}

void	LinuxEnableInterrupts(void *data)
{
	struct pci_dev *dev = (struct pci_dev *)data;
//	printk("<1>Test: LinuxEnableInterrupts 0x%x\n",dev->irq);
	enable_irq(dev->irq);
}

void	LinuxDisableInterrupts(void *data)
{
	struct pci_dev *dev = (struct pci_dev *)data;
	disable_irq(dev->irq);
}

void	LinuxMarkBH(void (*BHroutine)(void *), void *BHdata)
{
	basic2BH.routine = BHroutine;
	basic2BH.data = BHdata;
/*  	printk("LinuxMarkBH...\n"); */
	queue_task(&basic2BH, &tq_immediate);
	mark_bh(IMMEDIATE_BH);
}


unsigned short LinuxReadw(void *addr)
{
	unsigned short val = readw(addr);
/*  	printk("<1>Test: LinuxReadw read 0x%x from 0x%x\n",val,addr);  */
	return val;
}

unsigned int LinuxReadl(void *addr)
{
	unsigned int val = readl(addr);
/*  	printk("LinuxRead read 0x%x from 0x%x\n",val,addr); */
	return (val);
}

void	LinuxWritew(void *addr, unsigned short wVal)
{
/*  	printk("<1>Test: LinuxWritew writes 0x%x to 0x%x\n",wVal,addr); */
	writew(wVal, addr);
}
 
 
void	LinuxWritel(void * addr, unsigned int dwVal)
{
/*  	printk("<1>Test:LinuxWritel(%x, %x)\n",addr, dwVal); */
	writel(dwVal, addr);
}
 
 
unsigned char LinuxIoReadb(unsigned int port)
{
	unsigned char val = inb(port);
/*  	printk("<1>Test: LinuxIoReadb read 0x%x from 0x%x\n",val,port);  */
	return val;
}
 
 
unsigned short LinuxIoReadw(unsigned int port)
{
	unsigned short val = inw(port);
/*  	printk("<1>Test: LinuxIoReadw read 0x%x from 0x%x\n",val,port);  */
	return val;
}
 
unsigned int LinuxIoReadl(unsigned int port)
{
	unsigned int val = inl(port);
/*  	printk("<1>Test: LinuxIoReadw read 0x%x from 0x%x\n",val,port);  */
	return val;
}
 
 
void	LinuxIoWriteb(unsigned int port, unsigned char bVal)
{
/*  	printk("<1>Test: LinuxIoWritew writes 0x%x to 0x%x\n",bVal,addr); */
	outb(bVal, port);
}
 
void	LinuxIoWritew(unsigned int port, unsigned short wVal)
{
/*  	printk("<1>Test: LinuxIoWritew writes 0x%x to 0x%x\n",wVal,addr); */
	outw(wVal, port);
}
 
 
void	LinuxIoWritel(unsigned int port, unsigned int dwVal)
{
/*  	printk("<1>Test:LinuxIoWritel(%x, %x)\n",addr, dwVal); */
	outl(dwVal, port);
}
 
void LinuxPciReadConfigdw(struct pci_dev *PciDev,unsigned char Offset ,unsigned long *pVal)
{
	pci_read_config_dword(PciDev,Offset,(unsigned int *)pVal);
}
 
void LinuxPciWriteConfigdw(struct pci_dev *PciDev,unsigned char Offset ,unsigned long Val)
{
	pci_write_config_dword(PciDev,Offset,Val);
}
 
void 	LinuxSetPciBusMaster(void *data)
{
	struct pci_dev *dev = (struct pci_dev *)data;
	pci_set_master(dev);
}


unsigned long crit_section_start()
{
	unsigned long flags;
	save_flags(flags);
	cli();
	return flags;
}

void crit_section_end(unsigned long flags)
{
	restore_flags(flags);
}
 
 
unsigned long LinuxCalcCpuRate(void)
{
	struct timeval tv;
	unsigned long Flag;
	unsigned long time1, time2;
	unsigned int target_usec;
	
 
	// first verify that the tv_usec will not wraparound
//	gettimeofday( &tv, NULL );
	do_gettimeofday( &tv );
 
	if ( tv.tv_usec > 990000 ) {
		do
		{
//			gettimeofday( &tv, NULL );
			do_gettimeofday( &tv );
		} while ( tv.tv_usec > 990000 );
	}
 
	// calc the cpu rate 
	save_flags(Flag);
	cli();
//	gettimeofday( &tv, NULL );
	do_gettimeofday( &tv );
	time1 = ReadCpuCnt();
	target_usec = tv.tv_usec + 1000;
	do
	{
//		gettimeofday( &tv, NULL );
		do_gettimeofday( &tv );
	} while ( tv.tv_usec < target_usec );
	time2 = ReadCpuCnt();
	restore_flags( Flag );
 
	return( ( (time2-time1+500) /1000 ) * 1000000); // to get whole number 
}
 
