#include "ostypedefs.h"
#include "osdbg_ex.h"
#include "osdbg_in.h"
#include "osmemory_ex.h"
#include "osstring_ex.h"
//#include "klower.h"

#define DBG_VERSION                 30

/*************** DMP interface ************************/
I_DMP_SERVER IDmpServer = 
{
    DmpServer_SetIdleState,
    DmpServer_CreateClient,
    DmpServer_OpenClient,
    DmpServer_ConfigureClient,
    DmpServer_CloseClient,
    DmpServer_DestroyClient,
    DmpServer_WriteClient,
};

GLOBAL	PVOID	DmpSrvGetInterface(void)
{
    return &IDmpServer;
}

BOOL    DmpServer_SetIdleState(int IdleState)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	ALLREGS		AllRegs;
	PDDB pDDB = Get_DDB(0, "DMPSRV  " );
	if ( pDDB == NULL ) {
		return FALSE;
	} 

	memset( &AllRegs, 0, sizeof(ALLREGS) );
	AllRegs.RECX = (DWORD)pDDB;
	AllRegs.REAX = DMP_SET_IDLE_STATE;
	AllRegs.REBX = (DWORD)NULL;
	AllRegs.REDX = (DWORD)IdleState;

	Directed_Sys_Control( pDDB, DMP_SET_IDLE_STATE, &AllRegs );
#endif
	return TRUE;
}
HANDLE DmpServer_CreateClient()
{
    DMPSRV_CLIENT_INFO_T   *pClientInfo;
    pClientInfo = OsHeapAllocate( sizeof(DMPSRV_CLIENT_INFO_T), OS_NON_PAGED);
    if ( NULL != pClientInfo)
    {
        OsMemSet(pClientInfo, 0, sizeof(DMPSRV_CLIENT_INFO_T));
    }
    return pClientInfo;
}
BOOL   DmpServer_ConfigureClient(
                                 HANDLE                 hClient, 
                                 DMPSRV_CONFIG_CODES_T  ConfigCode, 
                                 PVOID                  pConfig)
{
    DMPSRV_CLIENT_INFO_T   *pClientInfo = (DMPSRV_CLIENT_INFO_T*)hClient;
    switch(ConfigCode)
    {
    case DMPSRV_CONFIG_FILE_PATH:
        OsStrCpy(pClientInfo->Path, (char *)pConfig);
        break;
    case DMP_CONFIG_FILTER_NAME:
        OsStrCpy(pClientInfo->FilterName, (char *)pConfig);
        break;
    }
    return TRUE;

}

BOOL DmpServer_OpenClient(HANDLE hClient)
{
    DMPSRV_CLIENT_INFO_T   *pClientInfo = (DMPSRV_CLIENT_INFO_T*)hClient;
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
    if ( FALSE == FindServer(pClientInfo, pClientInfo->FilterName) )
    {
        pClientInfo->pServerFunc = NULL;
    }
    else
    {
	    (*pClientInfo->pServerFunc)( pClientInfo->FilterName, DMP_CREATE_FILE, pClientInfo->Path, NULL, 0 );
	    if ( FALSE == PageAllocate( SEGMENT_SIZE/DMP_PAGE_SIZE, PG_SYS, 0, 0, 0, 0, &pClientInfo->PhysAddr, 
			    PAGELOCKED | PAGEZEROINIT, &pClientInfo->hData, (PPVOID)&pClientInfo->pSegment ))
        {
            pClientInfo->hData = 0;
            pClientInfo->PhysAddr = NULL;
            pClientInfo->pSegment = NULL;
        }
    }
#endif
#if ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER)) 
    pClientInfo -> pFile = fopen(pClientInfo->Path, "wb");
#endif
    return TRUE;
}
VOID   DmpServer_CloseClient(HANDLE hClient)
{
    DMPSRV_CLIENT_INFO_T   *pClientInfo = (DMPSRV_CLIENT_INFO_T*)hClient;
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
    if ( NULL != pClientInfo->pServerFunc )
    {
	    (*pClientInfo->pServerFunc)( pClientInfo->FilterName, DMP_CLOSE_FILE, pClientInfo->Path, NULL, 0 );
	    DisconnectServer(pClientInfo);
        if ( NULL!= pClientInfo->hData )
        {
        	PageFree( (MEMHANDLE)pClientInfo->hData, 0);
        }
    }
#endif
#if ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER)) 
    if (NULL != pClientInfo -> pFile  )
        fclose(pClientInfo -> pFile);
#endif
}
VOID    DmpServer_DestroyClient(HANDLE hClient)
{
    if ( NULL == hClient)
        return;
    OsHeapFree(hClient);
}
int DmpServer_WriteClient(HANDLE hClient, BYTE *pBuffer, int nData)
{
    DMPSRV_CLIENT_INFO_T   *pClientInfo = (DMPSRV_CLIENT_INFO_T*)hClient;
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	UINT32 DataThisLoop, Data2Copy, DataCopied;
    if ( NULL == pClientInfo->pServerFunc || pClientInfo->hData == NULL)
    { 
        return 0;
    }
	
	Data2Copy = nData;
	DataCopied = 0;

	do {
		DataThisLoop = MIN( SEGMENT_SIZE-pClientInfo->SegmentCount, Data2Copy );// take only as much as possible
				
		OsMemCpy( pClientInfo->pSegment + pClientInfo->SegmentCount, pBuffer + DataCopied, DataThisLoop);
		pClientInfo->SegmentCount += DataThisLoop;
		DataCopied += DataThisLoop;

		if ( DataThisLoop <= Data2Copy ) {
			if ( DumpToServer(pClientInfo) == FALSE )
				return 0;
		}

		Data2Copy -= DataThisLoop;
	} while ( Data2Copy > 0 );

	return nData; 
#endif
#if ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER)) 
    if (NULL != pClientInfo -> pFile  )
        return fwrite(pBuffer, sizeof(BYTE), nData, pClientInfo -> pFile);
    return 0;
#endif
}

#if  ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) ) 

BOOL FindServer(DMPSRV_CLIENT_INFO_T   *pClientInfo, PCHAR ClientName)
{
//#ifdef VXD

	ALLREGS		AllRegs;

	PDDB pDDB = Get_DDB(0, "DMPSRV  " );
	if ( pDDB == NULL ) {
		return FALSE;
	} 


	memset( &AllRegs, 0, sizeof(ALLREGS) );
	AllRegs.RECX = (DWORD)pDDB;
	AllRegs.REAX = DMP_SET_ENTRY_POINT_CALLBACK;
	AllRegs.REBX = (DWORD)&pClientInfo->pServerFunc;
	AllRegs.REDX = (DWORD)pClientInfo->FilterName;

	Directed_Sys_Control( pDDB, DMP_SET_ENTRY_POINT_CALLBACK, &AllRegs );

	if ( pClientInfo->pServerFunc == NULL )
		return FALSE;

	return TRUE;
//#endif

}
BOOL DisconnectServer(DMPSRV_CLIENT_INFO_T   *pClientInfo)
{
//#ifdef VXD
	PDDB pDDB = Get_DDB(0, "DMPSRV  " );
	ALLREGS		AllRegs;	
	if ( pDDB == NULL ) {
		pClientInfo->pServerFunc = NULL;
		return FALSE;
	} 


	memset( &AllRegs, 0, sizeof(ALLREGS) );
	AllRegs.RECX = (DWORD)pDDB;
	AllRegs.REAX = DMP_DELETE_ENTRY_POINT_CALLBACK;
	AllRegs.REDX = (DWORD)pClientInfo->FilterName;	
	Directed_Sys_Control( pDDB, DMP_DELETE_ENTRY_POINT_CALLBACK, &AllRegs );

	pClientInfo->pServerFunc = NULL;

	return TRUE;
//#endif
}

BOOL DumpToServer(DMPSRV_CLIENT_INFO_T   *pClientInfo)
{
	if ( pClientInfo->pServerFunc == NULL )
		return FALSE;

	if ( (*pClientInfo->pServerFunc)( pClientInfo->FilterName, DMP_WRITE_TO_FILE, pClientInfo->Path, (PBYTE)pClientInfo->pSegment, pClientInfo->SegmentCount) == FALSE ) {
		return FALSE;
	}

	pClientInfo->SegmentCount= 0;

	return TRUE;
}

#endif
/*************** end of DMP interface ************************/

/*************** SCR interface ************************/
//DbgDll.c
#include "intfctrl_ex.h"
#if ( ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) ) || ((OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL) ) )

#include "vxdcomm.h"
//#ifdef VXD
#if ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) ) 
	#define SCR_SET_ENTRY_POINT_CALLBACK	(DWORD)(BEGIN_RESERVED_PRIVATE_SYSTEM_CONTROL+1)
#endif

//#ifdef WDM
#if ((OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL) ) 
	#define SCRSRV_DEVICE L"\\Device\\DeviceScrSrv0"
	#define SCR_SET_ENTRY_POINT_CALLBACK	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80E, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif

int TotalClients = 0;
I_SCR_SRV	IScrSrv;
//#ifdef VXD
#if ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) ) 
PDDB		DbgDevice_pDBB;
#endif
PSCR_PRINT_FUNC pPrintFunc = NULL;
CHAR gVxdName[100];

VOID	GetDbgServerEntryPoint(PVOID pParam)
{
	pPrintFunc = (PSCR_PRINT_FUNC)pParam;
}

void __fastcall PrintFunc(PCHAR Str, PRIORITY_DESCRIPTOR PriorityDescriptor)
{
	if ( NULL != pPrintFunc )
	{
		pPrintFunc(Str, PriorityDescriptor);
	}

}

VOID GetMyVxdName()
{
#if VXD
    int i;
	if ( !GetVxDName((DWORD)GetMyVxdName , gVxdName) )
		return;

	for(i=0; i < 8; i++)
		if (  gVxdName[i] == '(' )
			break;
	for(;i<8;i++)
		gVxdName[i] = ' ';	
	gVxdName[8] = '\0';
#endif
}

void SetOctopusClientInfo(PSET_CLIENT_INFO_CALLBACK pClientCallBack, HANDLE hClient)
{
//#ifdef VXD
#if ((OS_TYPE == OS_WIN_9X)  && (UK_MODE == UK_KERNEL) ) 
    if ( NULL != pClientCallBack )
    {
        TotalClients++;
        if ( 1 == TotalClients)
		{
	        ALLREGS		AllRegs;
            GetMyVxdName();
	        DbgDevice_pDBB = Get_DDB( 0, "MDMONSRV  " );
	        if ( NULL == DbgDevice_pDBB )
		        return ;
	        memset( &AllRegs, 0, sizeof(ALLREGS) );
	        AllRegs.RECX = (DWORD)DbgDevice_pDBB;
	        AllRegs.REAX = SCR_SET_ENTRY_POINT_CALLBACK;
	        AllRegs.REBX = (DWORD)GetDbgServerEntryPoint;
	        AllRegs.REDX = (DWORD)gVxdName;

	        Directed_Sys_Control( DbgDevice_pDBB, SCR_SET_ENTRY_POINT_CALLBACK, &AllRegs );
        }        
        pClientCallBack(hClient, PrintFunc);
    }
    else
    {
        TotalClients--;
    }

#endif

//#ifdef WDM	
#if ((OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL) ) 
	ULONG NumBytes = 0;
	DWORD Version = DBG_VERSION;
	NTSTATUS Status;
	KIRQL CurIrql;
	DWORD InfBuf [2];
	InfBuf[0] = GetDbgServerEntryPoint;//pClientInfo->pServerFunc;//(DWORD)Func;
	//InfBuf[1] = (DWORD)this;
		
	if ( NULL != pClientCallBack )
    {
        TotalClients++;
        if ( 1 == TotalClients)
        {	
			CurIrql = KeGetCurrentIrql();
			if (CurIrql != PASSIVE_LEVEL)
			{
				DbgPrint("DbgScr: Can't connect to DbgApp server (IRQL = %d)", (DWORD)CurIrql);
				return;
			}
			construct(SCRSRV_DEVICE, FILE_ALL_ACCESS );

			if ( ! NT_SUCCESS(/* pScrSrv ->*/ GetConstructorStatus() ) )
			{						
				return;
			}
									
			Status = /*pScrSrv ->*/ DeviceIoControl ( SCR_SET_ENTRY_POINT_CALLBACK,
	  					  							 (PVOID) InfBuf,
													 2 * (sizeof(DWORD)),
													 (PVOID) &Version,
													 sizeof (DWORD),
													 FALSE,
													 &NumBytes);


			if (NT_SUCCESS(Status))
			{
			//	if (pClientInfo->pServerFunc == NULL)
				if (pPrintFunc == NULL)			
					GetDbgServerEntryPoint (NULL/*pClientInfo->pServerFunc*/);				
			}
			else 
			{
				return;
			}

		}
        
        pClientCallBack(hClient, PrintFunc);
    }
    else
    {
        TotalClients--;
		if (NULL != pObject)
			OsHeapFree(pObject);
    }
	
#endif

}

PVOID ScrSrvGetInterface()
{
	IScrSrv.SetClientInfo = SetOctopusClientInfo;
	return &IScrSrv;
}

#endif /* ((OS_TYPE == OS_WIN_9X)  || (UK_MODE == UK_KERNEL) ) */

#if ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )

#include "cd_scr.h"
HWND	 hDbgScr;
int		nClients;
static char					sMessage[MAX_TOTAL_MESSAGE_LEN];

typedef	struct SCR_CLIENT_INFO_TAG
{
	HANDLE						hClient;
	PSET_CLIENT_INFO_CALLBACK	ClientFunc;
}SCR_CLIENT_INFO, *PSCR_CLIENT_INFO;
	
I_SCR_SRV	IScrSrv;
VOID  __fastcall DBGDLL_Print(char *Message)
{

	COPYDATASTRUCT	Data;
	strncpy(sMessage, Message, MAX_TOTAL_MESSAGE_LEN);
	Data.dwData = DBG_GET_EX_MESSAGE;
	Data.cbData = strlen(Message);
	Data.lpData = (LPVOID)sMessage;
	SendMessage(hDbgScr,WM_COPYDATA, 0,
			(LPARAM) (PCOPYDATASTRUCT)&Data);
}
void SetOctopusClientInfo(PSET_CLIENT_INFO_CALLBACK pClientCallBack, HANDLE hClient)
{
    if ( NULL != pClientCallBack)
    {
	    // send the vxd the DBGSRV_Print function pointer for use
	    pClientCallBack(hClient, DBGDLL_Print);
    }

}

PVOID ScrSrvGetInterface()
{
	char Apptitle [255];
	sprintf (Apptitle, "%s - Conexant Confidential",APP_NAME );
	hDbgScr = FindWindow(NULL,Apptitle);
	if (!hDbgScr) {
		return (int)NULL;
	}
	IScrSrv.SetClientInfo = SetOctopusClientInfo;
	return &IScrSrv;
}
#endif/* ((OS_TYPE == OS_WIN_9X) || (UK_MODE == UK_USER) ) */


#if ( (OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL) ) 
VOID construct(IN PCWSTR name, IN ACCESS_MASK access)
{
	NTSTATUS nt;
	UNICODE_STRING u;
	pFileObject = NULL;
	pDeviceObject = NULL;

	
	RtlInitUnicodeString(&u, name);

	nt = ConstructorStatus = 
		IoGetDeviceObjectPointer(&u, access, &pFileObject, &pDeviceObject);
	
}


NTSTATUS GetConstructorStatus(void)
{
	return ConstructorStatus;
}

NTSTATUS DeviceIoControl(
	ULONG IoControlCode,
	PVOID InputBuffer,
	ULONG InputBufferSize,
	PVOID OutputBuffer,
	ULONG OutputBufferLength,
	BOOLEAN Internal,
	ULONG* pInformation
	)
{
	PIRP pIrp;
	IO_STATUS_BLOCK StatusBlock;
	NTSTATUS status;

	//KEvent event(NotificationEvent, FALSE);
	KEvent(NotificationEvent, FALSE);

	pIrp = IoBuildDeviceIoControlRequest(
		IoControlCode,
		pDeviceObject,
		InputBuffer,
		InputBufferSize,
		OutputBuffer,
		OutputBufferLength,
		Internal,
//		(PKEVENT)event.Object(),
		(PKEVENT)pObject,
		&StatusBlock);

	if (pIrp == NULL)
		return STATUS_INSUFFICIENT_RESOURCES;

//	BOUNDS_CHECKER(CALL_LOWER_DEVICE, (this, pIrp));

	status = IoCallDriver(pDeviceObject, pIrp);

	if (status == STATUS_PENDING)
	{
//		event.Wait();
		KeWaitForSingleObject(	pObject,
								Executive,
								KernelMode,
								TRUE,
								NULL);
		status = StatusBlock.Status;
	}

	*pInformation = StatusBlock.Information;

//	BOUNDS_CHECKER(COMPLETED_CALLDOWN, (this, status, *pInformation));

	return status;
	
}

KEvent(EVENT_TYPE type, BOOLEAN signaled)
{
	pObject = OsHeapAllocate( sizeof(KEVENT), OS_NON_PAGED);
	ASSERT( KeGetCurrentIrql() < DISPATCH_LEVEL );
	KeInitializeEvent((PKEVENT)pObject, type, signaled);
	CreatedByName = FALSE;
	
//	pObject = (PVOID) new(NonPagedPool) CHAR[size];
//	m_Handle = (HANDLE)-1;
}


#endif




/*************** end of SCR interface ************************/



