#include "ostypedefs.h"
#include "osfile_ex.h"
#include "osstring_ex.h"



#if (OS_TYPE == OS_WIN_WDM) 
#include "fileiowdm.h"
#endif

#if (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
#include "osmemory_ex.h"
typedef struct {
	HANDLE			hFile;
	UINT32			Index;
	UINT16			Mode;
} FILE;
#endif (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
/********************************************************************************

Function Name: OsFileOpen.

Function Description:Return. 

Arguments: Filename - Filename.
		   Mode - FILE_MODE_READ/FILE_MODE_WRITE/FILE_MODE_BINARY

Return Value: A handle to an opened file

********************************************************************************/
HANDLE	OsFileOpen(IN char *Filename, DWORD Mode)
{
#if (UK_MODE==UK_USER)
    char cMode[10] = "";
    if ( Mode & FILE_MODE_READ )
    {
        OsStrCat(cMode, "r");
    }
    if ( Mode & FILE_MODE_WRITE )
    {
        OsStrCat(cMode, "w");
    }
    if ( Mode & FILE_MODE_BINARY )
    {
        OsStrCat(cMode, "b");
    }
    if ( Mode & FILE_MODE_TEXT )
    {
        OsStrCat(cMode, "t");
    }
    return (HANDLE)fopen(Filename, cMode);
#endif //(UK_MODE==UK_USER)
#if (OS_TYPE == OS_WIN_WDM) 
    HANDLE      RetHandle;
    BOOL        bRead;
    if ( Mode & FILE_MODE_TEXT ) /* not supported */
    {
        return NULL;
    }
    if ( Mode & FILE_MODE_READ )
    {
        bRead = TRUE;
    }
    if ( Mode & FILE_MODE_WRITE )
    {
        bRead = FALSE;
    }
    OpenFile(Filename, bRead, &RetHandle); 
    return RetHandle;
#endif //(OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL)
#if (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
	FILE*	pFile;
	UINT16	Error;
	BYTE	Action;

	if ( Filename == NULL )
		return NULL;

	pFile = OsHeapAllocate(sizeof(FILE), OS_NON_PAGED);

	pFile->Index = 0;

	Error = 0;
    if ( Mode & FILE_MODE_WRITE )
    {
	    pFile->hFile = R0_OpenCreateFile( FALSE, (char *)Filename, OPEN_ACCESS_WRITEONLY, ATTR_NORMAL, 
		    ACTION_IFNOTEXISTS_CREATE | ACTION_IFEXISTS_TRUNCATE 
		    , R0_NO_CACHE,  &Error, &Action );
    }
    else if ( Mode & FILE_MODE_READ)
    {
	    pFile->hFile = R0_OpenCreateFile( FALSE, (char *)Filename, OPEN_ACCESS_READONLY, 0, 
		    ACTION_IFEXISTS_OPEN | ACTION_IFNOTEXISTS_FAIL 
		    , R0_NO_CACHE,  &Error, &Action );
    }
	if ( Error != 0 || pFile->hFile == 0 ) {
		OsHeapFree(pFile);
		return NULL;
	}

	return pFile;

#endif (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
}

/********************************************************************************

Function Name: OsFileClose.

Function Description: Closes a file. 

Arguments: hFile - handle to a file.

Return Value: None

********************************************************************************/
VOID	OsFileClose(HANDLE hFile)
{
#if (UK_MODE==UK_USER)
    fclose(hFile);
#endif
#if (OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL)
    if (NULL == hFile )
        return ;
    CloseFile(hFile);
#endif
#if (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
	UINT16	Error;
    FILE *pFile = (FILE *)hFile;
	if ( pFile == NULL || pFile->hFile == 0 )
		return 0;

	R0_CloseFile(pFile->hFile, &Error);
    OsHeapFree(pFile);
#endif (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
}

/********************************************************************************

Function Name: OsFileWrite.

Function Description:Writes a buffer . 

Arguments: 
    hFile - handle to a file.
    pBuffer - Pointer to data to be written
    Size - Item size in bytes
    count - Maximum number of items to be written



Return Value: returns the number of full items actually written

********************************************************************************/
int	OsFileWrite(
                HANDLE hFile, 
                IN  VOID *pBuffer, 
                IN  int Size, 
                IN  int count)
{
#if (UK_MODE==UK_USER)
    return fwrite(pBuffer, Size, count, hFile);
#endif (UK_MODE==UK_USER)
#if (OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL)
    NTSTATUS    Status;
    ULONG       RetBytes;
    if (NULL == hFile )
        return 0;
    Status = WriteFile(hFile, pBuffer, Size * count, &RetBytes);
    if ( STATUS_SUCCESS == Status)
        return RetBytes;
    else
        return 0;
#endif
#if (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
	UINT16	Error;
    UINT32 BytesRead;
    FILE *pFile = (FILE *)hFile;
	if ( pFile == NULL || pFile->hFile == 0 )
		return 0;


	BytesRead = R0_WriteFile( FALSE, pFile->hFile, (void *)pBuffer, Size * count ,
									pFile->Index, &Error );
	if ( Error != 0 )
		return 0;

	pFile->Index += BytesRead;

	return BytesRead/Size;
#endif (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
}

/********************************************************************************

Function Name: OsFileRead.

Function Description:Writes a buffer . 

Arguments: 
    hFile - handle to a file.
    pBuffer - Pointer to data to be read
    Size - Item size in bytes
    count - Maximum number of items to be read



Return Value: returns the number of full items actually read

********************************************************************************/
int	OsFileRead(
               IN   HANDLE hFile, 
               OUT  VOID *pBuffer, 
               IN   int Size, 
               IN   int count)
{
#if (UK_MODE==UK_USER)
    return fread( pBuffer, Size, count, hFile);
#endif (UK_MODE==UK_USER)
#if (OS_TYPE == OS_WIN_WDM) && (UK_MODE == UK_KERNEL)
    NTSTATUS    Status;
    ULONG       RetBytes;
    if (NULL == hFile )
        return 0;
    Status = ReadFile(hFile, pBuffer, Size * count, &RetBytes);
    if ( STATUS_SUCCESS== Status)
        return RetBytes;
    else
        return 0;
#endif
#if (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)

    UINT16	Error;
    FILE *pFile = (FILE *)hFile;
    UINT32 BytesRead;
	if ( pFile == NULL || pFile->hFile == 0 )
		return 0;


	BytesRead = R0_ReadFile( FALSE, pFile->hFile, pBuffer,
				       Size*count, pFile->Index, &Error );
	if ( Error != 0 )	{
		return 0;
	}

	pFile->Index += BytesRead;
	
	return BytesRead/Size;
#endif (OS_TYPE ==  OS_WIN_9X) && (UK_MODE == UK_KERNEL)
}

