/*************************************************************************

	File Name: Memory.c

	File Description:

*************************************************************************/
#if (OS_TYPE != OS_LINUX)
#include "common/include/framewrk.h"
#include "common/include/ossysenv.h"
#include "octopus/include/osmemory_ex.h"
#elif ( OS_TYPE == OS_LINUX )
#include "framewrk.h"
#include "ossysenv.h"
#include <linux/malloc.h>
#include <asm/io.h>
#include "osmemory_ex.h"
#endif



/******************************************************************************
	
	Function Name: OsPageCommit.

	Function Description:The function commits a memory space.

	Arguments: iPages - the base address of the memory space.
			   nPages - number of pages to be commited.
	
	Return Value: BOOL.

******************************************************************************/
 
BOOL OsPageCommit(UINT32 iPage, UINT32 nPages,UINT32  PageData, UINT32 flags)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		return PageCommit(iPage, nPages, (PAGERHANDLE)PageData, 0, flags | PC_USER);
#endif
#if ( ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) ) || \
      ( OS_TYPE == OS_LINUX ) || (OS_TYPE == OS_DOS) )
		return TRUE;
#endif
}
 
/******************************************************************************
	
	Function Name: OsPageDecommit.

	Function Description:The function decommits a given memory space.

	Arguments: iPages - the base address of the memory space.
			   nPages - number of pages to be commited.
	
	Return Value: BOOL.

******************************************************************************/
BOOL OsPageDecommit(UINT32 iPage, UINT32 nPages) 
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		return PageDecommit(iPage, nPages, 0);
#endif
#if ( ((OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER)) || \
      ( OS_TYPE == OS_LINUX ) || (OS_TYPE == OS_DOS))
		return TRUE;
#endif
}
 
/******************************************************************************
	
	Function Name: OsPageReserve.

	Function Description:The function decommits a given memory space.

	Arguments: iPages - the base address of the memory space.
			   nPages - number of pages to be commited.
	

	Return Value: a pointer to the reserved memory space.

******************************************************************************/
PVOID OsPageReserve(UINT32 nPages, UINT32 Flags)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		return PageReserve(PR_SYSTEM,nPages,PR_FIXED);
#endif
#if ( ((OS_TYPE == OS_WIN_9X)|| (OS_TYPE == OS_DOS)) && (UK_MODE == UK_USER) )
		return OsHeapAllocate(nPages * (1 << 12), 0);
#endif
#if ( OS_TYPE == OS_LINUX )
/*  		int i; */
/*  		PVOID temp = NULL; */
/*  		if ( 0 == nPages ) */
/*  		{ */
/*  			return (temp); */
/*  		} */
/*  		for (i=0;i<5;i++) */
/*  		{ */
/*  			if ((0x1<<i) > nPages) */
/*  			{ */
/*  				break; */
/*  			} */
/*  		} */
/*  		temp = (PVOID)__get_free_pages(Flags,i); */
		PVOID temp = NULL;
		temp = vmalloc(nPages*4096);
		if (NULL != temp)
			OsMemSet(temp,0,nPages*4096);
		return (temp);
#endif
}
 
/******************************************************************************
	
	Function Name: OsPageFree.

	Function Description:The function frees a given memory space.

	Arguments: iPages - the base address of the memory space.
			   nPages - number of pages to free.
	

	Return Value: BOOL

******************************************************************************/
BOOL OsPageFree(PVOID	pMem, UINT32 nPages)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		return PageFree((MEMHANDLE)pMem,0);
#endif
#if ( ((OS_TYPE == OS_WIN_9X)|| (OS_TYPE == OS_DOS)) && (UK_MODE == UK_USER) )
		if ( NULL == pMem )
			return FALSE;
		OsHeapFree ( pMem );
		return TRUE;
#endif
#if ( OS_TYPE == OS_LINUX )
/*  		int i; */
		if ( NULL == pMem )
			return FALSE;
/*  		for (i=0;i<5;i++) */
/*  		{ */
/*  			if ((0x1<<i) > nPages) */
/*  			{ */
/*  				i--; */
/*  				break; */
/*  			} */
/*  		} */
/*  		free_pages((UINT32)pMem, i); */
		vfree(pMem);
		return TRUE;
#endif
}
 
/******************************************************************************
	
	Function Name: OsHeapAllocate.

	Function Description:The function allocates a given memory space and initializes it to zero.

	Arguments: Size - the size of memory space (heap) to br allocated.
	
	Return Value: a pointer to the allocated memory..

******************************************************************************/
PVOID OsHeapAllocate(UINT32 Size, int flags)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		return HeapAllocate(Size,flags | HEAPZEROINIT);
#else
#if ( ((OS_TYPE == OS_WIN_9X) || (OS_TYPE == OS_DOS) )&& (UK_MODE == UK_USER))
		PVOID pMem = malloc(Size);
#endif
#if ( OS_TYPE == OS_LINUX )
		PVOID pMem = kmalloc(Size,flags);
#endif
		if ( NULL != pMem )
			memset(pMem, 0, Size);
		return pMem;
#endif
}
 
/******************************************************************************
	
	Function Name: OsHeapFree.

	Function Description:The function frees a memory space.

	Arguments: pMem - a pointer to the memory to be frees.
	
	Return Value: None.

******************************************************************************/
void OsHeapFree(PVOID pMem)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		HeapFree(pMem,0);
#endif
#if ( ((OS_TYPE == OS_WIN_9X) || (OS_TYPE == OS_DOS)) && (UK_MODE == UK_USER) )
		free(pMem);
#endif
#if ( OS_TYPE == OS_LINUX )
		kfree(pMem);
#endif
}
	
/******************************************************************************
 
	Function Name: OsGetPageSize.

	Function Description:The function returns the size of a page in the specific OS.

	Arguments: None.
	
	Return Value: The size of a page in the specific OS.

******************************************************************************/
UINT32	OsGetPageSize(void)
{
		return (1<<12);
}
/******************************************************************************
	
	Function Name: OsMemSet.

	Function Description:The function initializes a buffer to a specified character.

	Arguments: Buffer - Pointer to buffer to contain copied characters.
			   c - Character to copy into the buffer.
			   Count - Number of times to copy the character into the buffer.
	
	Return Value: A pointer to Buffer.

******************************************************************************/
PVOID	OsMemSet(PVOID Buffer, UINT32 c, UINT32 Count)
{
		return memset(Buffer, c, Count);
}
 
/******************************************************************************
	
	Function Name: OsMemMove.
 
	Function Description:The memmove function copies count bytes of characters from 
				src to dest. If some regions of the source area and the destination 
				overlap, memmove ensures that the original source bytes in the 
				overlapping region are copied before being overwritten.
	Arguments: dest - Destination object.
			   src - Source object
			   count - NNumber of bytes of characters to copy.
	
	Return Value: memmove returns the value of dest
 
******************************************************************************/
void *OsMemMove( void *dest, const void *src, int count )
{
		return memmove(dest, src, count);
}
/******************************************************************************
	
	Function Name: OsMemCpy.

	Function Description:The function copies a specified number of characters from one 
						 buffer to another. This function may not work correctly if 
						 the buffers overlap. 

	Arguments: BuffDest - Pointer to buffer to contain copied characters.
			   BuffSrc - Pointer to buffer with the characters to copy.
			   Count - Number of characters to copy.
	
	Return Value: A pointer to BuffDest.

******************************************************************************/
PVOID	OsMemCpy(PVOID BuffDest, PVOID BuffSrc, UINT32 Count)
{
		return memcpy(BuffDest, BuffSrc, Count);
}
 
/******************************************************************************
	
	Function Name: OsMemCmp.

	Function Description:The function compares a specified number of characters from two buffers.

	Arguments: Buff1 - Pointer to first buffer to compare.
			   Buff2 - Pointer to second buffer to compare.
			   Count - Number of characters to compare.
	
	Return Value: Returns one of three values as follows:
														0 if buffers match.
														< 0 if buff1 is less than buff2.
														>0 if buff1 is greater than buff2.

******************************************************************************/
int		OsMemCmp(PVOID Buff1, PVOID Buff2, UINT32 Count)
{
		return memcmp(Buff1, Buff2, Count);
}
 
/******************************************************************************
	
	Function Name: OsMemDMAAllocate.

	Function Description:The function allocate a dma.

	Arguments: nPages - number of page to allocate..
			   ppPhysAddr - a pointer to be filled with the physical address of the dma.
			   pMemHandle - a pointer to be filled with the memory handle.
			   ppBuffer - a pointer to be filled with the ring-0 linear memory address.
	
	Return Value: boolean.

******************************************************************************/

//typedef struct *DMAALLOCATION PDMAALLOCATION;

//typedef struct tag_DMA_ALLOCATION 
//{
//	PUINT16 pBuff;
//	struct tag_DMA_ALLOCATION *next;
//}DMAALLOCATION, *PDMAALLOCATION;

BOOL	OsMemDMAAllocate(UINT32 nPages,PUINT16* ppPhysAddr, PHANDLE pMemHandle, PUINT16* ppBuffer)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
    return PageAllocate(nPages, PG_SYS, 0, 0x0000001fL, 0x0, 0x1000,ppPhysAddr,
			PAGEFIXED | PAGEUSEALIGN | PAGEZEROINIT |
			PAGECONTIG | PAGELOCKED,
			(PMEMHANDLE) pMemHandle, (PPVOID) ppBuffer);
#endif
#if ( OS_TYPE == OS_LINUX )
	int count;
	PUINT16 allocs[256];

	for ( count = 0 ; count < 256 ; count++  )
	{
		allocs[count] = (PUINT16) kmalloc((1<<12), GFP_KERNEL | GFP_DMA);
		if ( allocs[count] == NULL )
		{
			break;
		}

		if ( 0x0 == (((UINT32)(allocs[count])) & 0x0000FFFF) )
		{
			*ppBuffer = allocs[count];
			*pMemHandle = (PVOID) allocs[count];
			*ppPhysAddr = (PUINT16)virt_to_bus(*ppBuffer);
//			printk("OsMemDMAAllocate success - 0x%x(0x%x) after %d attempts\n",
//				   (int) allocs[count], (int) *ppPhysAddr, count);
			for ( count--; count >= 0; count-- )
			{
				kfree(allocs[count]);
			}
			return (TRUE);
		}
	}

	printk("OsMemDMAAllocate failed\n");
	for ( count--; count >= 0; count-- )
	{
		kfree(allocs[count]);
	}
	return (FALSE);
#else
	return (TRUE);
#endif
}
 
/******************************************************************************
	
	Function Name: OsMemDMAFree.

	Function Description:The function allocate a dma.

	Arguments: MemHandle - a memory handle.
			   Flags - flags.
	
	Return Value: boolean.

******************************************************************************/
BOOL	OsMemDMAFree(HANDLE MemHandle, UINT32	Flags)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		return PageFree((MEMHANDLE)MemHandle, Flags);
#endif
#if ( OS_TYPE == OS_LINUX )
		if (NULL == MemHandle)
			return (FALSE);
//		printk( "OsMemDMAFree: freeing 0x%lx\n", MemHandle );
		kfree(MemHandle);
		return (TRUE);
#else
		return (TRUE);
#endif
}
