/*osTime.c

This file includes os specific code for timing related functionalities.
The code is to be compiled with VtoolsD 

*/
#include "framewrk.h"
#include "ossysenv.h"
#include "ostime_ex.h"
#include "ostime_in.h"
#if ( OS_TYPE == OS_LINUX )
//static DECLARE_TASK_QUEUE(tq_hsf); Not in use!
#endif
/*
This file includes os specific code for timing related functionalities.
The code is to be compiled with VtoolsD 
*/
/********************************************************************/
/* Construct a periodic time out instance.RunTime manager does not	*/
/* simply use global timers , since these are called with the		*/
/* same thread handle as the ring3 application. This prevent		*/
/* synchronization between passive and async calls with semaphores	*/
/* or mutexs.
/* Parameters :                                                     */
/* Irql - run time IQRL in which the timeout call back              */  
/*        should be called. In Wi95 this parameter in ignored,      */
/*        and all timeout are called at dispatch level.             */
/* InitialTimeout - Initial timeout interval in ms pRefData         */
/*                  should be passed as a parameter to this function*/
/* PTimeOutCallBack - CallBack to be call every timeout duration.   */
/* pFuncAlloc - alloc function,pRefData should be passed as         */ 
/*              parameter for thids function. function should       */
/*              allocate memory in non-paged pool                   */
/* pFuncFree - free function,pRefData should be passed as           */
/*             parameter for thids function                         */
/* pRefData -  reference data to be passed to pTimeOutCallBack      */
/*             pFuncAllc and pFuncFree                              */
/********************************************************************/
// @@@@ Irql unreferenced
#if ( COMPILER == CMP_MSDEV)
#pragma warning (disable:4100)
#endif
GLOBAL	HANDLE	OSCreatePeriodicTimeOut	(	IN	TIMER_IRQL_TYPE	Irql,
											IN	UINT32			InitialTimeOut,
											IN	PCBFUNC			pTimeOutCallBack,
											IN	PALLOC_FUNC		pFuncAlloc,
											IN	PFREE_FUNC		pFuncFree,
											IN	PVOID			pRefData)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	unsigned long		ThreadID;
#endif
	PTIME_OUT_INSTANCE_T	pTimeOutInstance;
	pTimeOutInstance= pFuncAlloc(sizeof(TIME_OUT_INSTANCE_T), pRefData);
	if (NULL == pTimeOutInstance )
		return(NULL);
	pTimeOutInstance->pFuncFree			= pFuncFree;
	pTimeOutInstance->pRefData			= pRefData;
	pTimeOutInstance->pTimeOutCallBack	= pTimeOutCallBack;
	pTimeOutInstance->bLocked			= FALSE;
	pTimeOutInstance->mSec				= InitialTimeOut;
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )

	pTimeOutInstance->TimeOutHandle		= (TIMEOUTHANDLE)NULL;
	pTimeOutInstance->hSemaphore		= Create_Semaphore(0);
	if ((SEMHANDLE)NULL == pTimeOutInstance->hSemaphore )
	{
		pFuncFree(pTimeOutInstance, pRefData);
		return (NULL);
	}

	pTimeOutInstance -> hThread = VWIN32_CreateRing0Thread(4096, (UINT32)pTimeOutInstance,
		TimerThreadFunction , 0, NULL);
	if ((THREADHANDLE)NULL == pTimeOutInstance -> hThread )
	{
		Destroy_Semaphore(pTimeOutInstance->hSemaphore);
		pFuncFree(pTimeOutInstance, pRefData);
		return (NULL);
	}


	return( (HANDLE)pTimeOutInstance);
#endif // OS_VXD
#if ( OS_TYPE == OS_LINUX )
 
//	queue_task(&(pTimeOutInstance -> TaskQueue), &tq_immediate);
	init_timer(&pTimeOutInstance -> Timer);
	pTimeOutInstance -> Timer.function = TimeOutHandler;
	pTimeOutInstance -> Timer.data = (unsigned long)pTimeOutInstance;
	pTimeOutInstance -> Timer.expires = jiffies + 10;
	pTimeOutInstance -> active = TRUE;
	pTimeOutInstance->Count = 0;
	add_timer(&(pTimeOutInstance -> Timer));

	return( (HANDLE)pTimeOutInstance);
#endif // OS_VXD
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	pTimeOutInstance->TimeOutHandle		= (UINT32)NULL;
	pTimeOutInstance->hSemaphore		= CreateSemaphore(
		NULL, 
		(LONG)0, 
		2, 
		NULL);
	if ((HANDLE)NULL == pTimeOutInstance->hSemaphore )
	{
		pFuncFree(pTimeOutInstance, pRefData);
		return (NULL);
	}
	pTimeOutInstance->hThread = CreateThread(
		NULL,
		2048,
		TimerThreadFunction,
		pTimeOutInstance,
		0,
		&ThreadID);
	if ((HANDLE)NULL == pTimeOutInstance -> hThread )
	{
		CloseHandle(pTimeOutInstance->hSemaphore);
	}

#endif // OS_APPLICATION_MODE
	return( (HANDLE)pTimeOutInstance);
	
}
#if ( COMPILER == CMP_MSDEV)
#pragma warning (default:4100)
#endif
/********************************************************************/
GLOBAL	VOID	OSDestoyPeriodicTimeOut	(IN	HANDLE	hTimeOut)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)hTimeOut;
	int						InWhile = 0;

	pTimeOutInstance->bLocked			= TRUE;
	pTimeOutInstance -> active = FALSE;
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	if ( (UINT32)NULL  != pTimeOutInstance->TimeOutHandle )
	{
		Cancel_Time_Out(pTimeOutInstance->TimeOutHandle);
	}
#endif //OS_VXD
#if ( OS_TYPE == OS_LINUX )
	while( pTimeOutInstance->Count > 0 )
	{
		InWhile++;
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )
		run_task_queue(&tq_scheduler);
#else
		flush_scheduled_tasks();
#endif
		if(InWhile>=10)
		{
			pTimeOutInstance->Count = 0;
		}
	}
	del_timer(&pTimeOutInstance -> Timer);
#endif //OS_LINUX
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	if ( NULL != pTimeOutInstance->hSemaphore		)
	{
		CloseHandle(pTimeOutInstance->hSemaphore);
	}
	if ( NULL != pTimeOutInstance->hThread )
	{
		CloseHandle(pTimeOutInstance->hThread );
	}
#endif //OS_APPLICATION_MODE

	pTimeOutInstance->pFuncFree(pTimeOutInstance,pTimeOutInstance->pRefData);

}
/********************************************************************/
GLOBAL	BOOL	OSSetPeriodicTimeOut		(	IN	HANDLE			hTimeOut,
												IN	UINT32			NewTimeOut)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)hTimeOut;
	BOOL					TimerIsActive = FALSE;
	pTimeOutInstance->mSec = NewTimeOut;
#if ( OS_TYPE == OS_LINUX )
	if ( timer_pending(&(pTimeOutInstance->Timer) ))
	{
		TimerIsActive = TRUE;
	}
	del_timer(&pTimeOutInstance -> Timer);
	init_timer(&pTimeOutInstance -> Timer);
	pTimeOutInstance -> Timer.function = TimeOutHandler;
	pTimeOutInstance -> Timer.data = (unsigned long)pTimeOutInstance;
	pTimeOutInstance -> Timer.expires = jiffies + ((pTimeOutInstance -> mSec)/10);
	 if ( TimerIsActive )
	 {
		 add_timer(&(pTimeOutInstance -> Timer));
	 }
	return TRUE;
#endif // OS_LINUX
#if ( OS_TYPE != OS_LINUX )
	if ( (UINT32)NULL != pTimeOutInstance -> TimeOutHandle )
	{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
		Cancel_Time_Out(pTimeOutInstance -> TimeOutHandle);
		pTimeOutInstance->TimeOutHandle		= Set_Global_Time_Out(
			pTimeOutInstance->mSec,
			pTimeOutInstance,
			TimeOutHandler,
			&(pTimeOutInstance->TimeOuthunk));
#endif // OS_VXD
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
		timeKillEvent(pTimeOutInstance->TimeOutHandle);
		pTimeOutInstance->TimeOutHandle		= timeSetEvent(
			NewTimeOut, 
			0, 
			(LPTIMECALLBACK)TimeOutHandler, 
			(UINT32)pTimeOutInstance,
			TIME_PERIODIC);
#endif // OS_APPLICATION_MODE
	}
	return ((UINT32)NULL != pTimeOutInstance->TimeOutHandle);
#endif
}

#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
/********************************************************************/
STATIC void __cdecl FuncOnNewStack(			IN	UINT32	lParam)
{
	PSTACK_CHANGE_INFO_T	pStackChangeInfo = (PSTACK_CHANGE_INFO_T)lParam;
	(pStackChangeInfo -> pFunc)((PVOID)pStackChangeInfo -> lParam);

}
#endif
/********************************************************************/
#if !( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
#if ( COMPILER == CMP_MSDEV)
#pragma warning (disable:4100)
#endif
#endif
volatile void *pstack;
 
GLOBAL	VOID	OSCallOnMyStack(			IN	PCBFUNC			pFunc, 
											IN	UINT32			lParam, 
											IN	PUINT32			pTopOfStack, 
											IN	UINT32			StackSizeBytes)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	STACK_CHANGE_INFO_T	StackChangeInfo;

	StackChangeInfo.pFunc = pFunc;
	StackChangeInfo.lParam = lParam;

    	/* Cannot call pFunc dirctly, since _Call_On_My_Stack requires
	   that the function will be declared using the C calling convention.
	   so we are calling pFunc through FuncOnNewStack function*/
	_Call_On_My_Stack((PVOID)FuncOnNewStack, (UINT32)&StackChangeInfo, pTopOfStack, StackSizeBytes);
#endif //OS_VXD
#if ( OS_TYPE == OS_LINUX )

/*  #ifndef CONFIG_SMP */
/*  	cli(); */
	asm volatile ( "mov %%esp, %0\n"
				   "mov %1, %%esp\n"
				   : "=r" (pstack) : "r" (pTopOfStack) );
/*  	sti(); */
	pFunc((PVOID)lParam); 
/*  	cli(); */
	asm volatile ( "mov %0, %%esp\n"
				   : "=r" (pstack) );
/*  	sti(); */
/*  #else */
/*  	  pFunc((PVOID)lParam); */
/*  #endif //CONFIG_SMP */

#endif //OS_LINUX
#if (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER)
	  pFunc(lParam);
#endif
}
#if !( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
#if ( COMPILER == CMP_MSDEV)
#pragma warning (default:4100)
#endif
#endif

#if ( OS_TYPE == OS_LINUX )
/********************************************************************/
STATIC VOID TimeOutHandler(	unsigned long Data )
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance;
	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)Data;
	if (pTimeOutInstance->active == FALSE )
	{
/*  		printk("TimeOutHandler 0x%x not active\n",(UINT32)pTimeOutInstance); */
		return;
	}
	pTimeOutInstance->Count++;
	pTimeOutInstance -> TaskQueue.routine = TimerThreadFunction;
	pTimeOutInstance -> TaskQueue.data = (PVOID)pTimeOutInstance;
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )
	queue_task(&(pTimeOutInstance -> TaskQueue), &tq_scheduler);
#else
	if ( schedule_task(&(pTimeOutInstance -> TaskQueue)) == 0)
	{
		printk("!!!!!! TimeOutHandler(%ld) - schedule_task FAILED !!!!!(%ld)\n",pTimeOutInstance->Count,jiffies);
	}
#endif
//	run_task_queue(&tq_hsf);
}
#endif // OS_LINUX

#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )

/********************************************************************/
STATIC VOID __stdcall TimeOutHandler(	IN	VMHANDLE hVM, 
										IN	PCLIENT_STRUCT pcrs, 
										IN	PVOID RefData, 
										UINT32 Extra)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance;
	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)RefData;
	pTimeOutInstance->TimeOutHandle = (TIMEOUTHANDLE)NULL;
	Signal_Semaphore(pTimeOutInstance->hSemaphore);
}
#endif // OS_VXD

#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
/********************************************************************/
#if ( COMPILER == CMP_MSDEV)
#pragma warning (disable:4100)
#endif
STATIC void CALLBACK   TimeOutHandler(	UINT	uID, 
									UINT	uMsg, 
									UINT32	dwUser,
									UINT32	dw1,     
									UINT32	dw2      )
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance;
	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)dwUser;
	pTimeOutInstance->TimeOutHandle = (UINT32)NULL;
	ReleaseSemaphore(pTimeOutInstance->hSemaphore, 1, NULL);
}
#if ( COMPILER == CMP_MSDEV)
#pragma warning (default:4100)
#endif
 
#endif // OS_APPLICATION_MODE

#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) ) 
/********************************************************************/
STATIC	VOID TimerThreadFunction(UINT32 hTimeOut)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance;
	if ( (UINT32)NULL == hTimeOut )
		return;
	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)hTimeOut;
	while(!pTimeOutInstance->bLocked )
	{
		pTimeOutInstance->TimeOutHandle		= Set_Global_Time_Out(
			pTimeOutInstance->mSec,
			pTimeOutInstance,
			TimeOutHandler,
			&pTimeOutInstance->TimeOuthunk);

		Wait_Semaphore(pTimeOutInstance->hSemaphore, BLOCK_SVC_INTS);
		pTimeOutInstance->pTimeOutCallBack(pTimeOutInstance->pRefData);
	}

}

#endif //OS_VXD
/********************************************************************/

#if ( OS_TYPE == OS_LINUX )
/********************************************************************/
 
 
STATIC	VOID TimerThreadFunction(PVOID pData)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)pData;
	if ( NULL == (PVOID)pTimeOutInstance )
		return;
	if (pTimeOutInstance->active == FALSE )
	{
		printk("TimerThreadFunction:: NOT ACTIVE!!!!\n");
	}
	init_timer(&pTimeOutInstance -> Timer);
	pTimeOutInstance -> Timer.function = TimeOutHandler;
	pTimeOutInstance -> Timer.data = (unsigned long)pTimeOutInstance;
	pTimeOutInstance -> Timer.expires = jiffies + ((pTimeOutInstance -> mSec)/10);
	pTimeOutInstance->pTimeOutCallBack(pTimeOutInstance->pRefData);
	add_timer(&(pTimeOutInstance -> Timer));
	pTimeOutInstance->Count--;
}

#endif //OS_LINUX 
/********************************************************************/

#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
STATIC unsigned long WINAPI TimerThreadFunction(  LPVOID lpParameter   )
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance;
	if ( (UINT32)NULL == lpParameter )
		return 0;
	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)lpParameter;
	while(!pTimeOutInstance->bLocked )
	{
		DPRINTF(("pTimeOutInstance->mSec = %d\n", pTimeOutInstance->mSec));
		pTimeOutInstance->TimeOutHandle		= timeSetEvent(
			pTimeOutInstance->mSec,
			0,
			(LPTIMECALLBACK)TimeOutHandler,
			(UINT32)pTimeOutInstance,
			TIME_ONESHOT);

		WaitForSingleObjectEx(pTimeOutInstance->hSemaphore, INFINITE, FALSE);
		pTimeOutInstance->pTimeOutCallBack(pTimeOutInstance->pRefData);
	}
	return 0;
	/*
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)lpParameter;
						
	pTimeOutInstance->TimeOutHandle		= timeSetEvent(
		pTimeOutInstance->mSec, 
		0, 
		TimeOutHandler, 
		(UINT32)pTimeOutInstance,
		TIME_PERIODIC);
	while(!pTimeOutInstance->bLocked )
	{
	}
	return 0;*/
}

#endif // OS_APPLICATION_MODE
/********************************************************************/
GLOBAL	UINT32	OSGetSystemTime(			VOID	)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	return( Get_System_Time());
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	return( GetTickCount());
#endif
#if ( OS_TYPE == OS_LINUX )
	struct timeval timestamp;
	do_gettimeofday(&timestamp);
	return ((UINT32)((timestamp.tv_sec*1000) + (timestamp.tv_usec/1000)));
#endif
}
/* Semaphores */
/********************************************************************/
GLOBAL HANDLE	OSSemaphoreCreate(	IN	int InitCount)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	return ( (HANDLE)Create_Semaphore((UINT32)InitCount));
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	return ( (HANDLE)CreateSemaphore(
		NULL, 
		(LONG)InitCount, 
		(1 + InitCount) * 10, 
		NULL));
#endif
#if ( OS_TYPE == OS_LINUX )
	struct semaphore *pSemaphore = kmalloc(sizeof(struct semaphore), GFP_KERNEL);
	if (NULL == pSemaphore)
	{
		return (NULL);
	}
	sema_init(pSemaphore, InitCount);
	return ((HANDLE)pSemaphore);
#endif
}

/********************************************************************/
GLOBAL VOID	OSSemaphoreDestroy(		IN	HANDLE hSemaphore)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	Destroy_Semaphore((UINT32)hSemaphore);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	CloseHandle(hSemaphore);
#endif
#if ( OS_TYPE == OS_LINUX )
	if (NULL != hSemaphore)
		kfree((void *)hSemaphore);
#endif
}

/********************************************************************/
GLOBAL VOID	OSSemaphoreWait(		IN	HANDLE hSemaphore)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	Wait_Semaphore((UINT32)hSemaphore, BLOCK_SVC_INTS);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	WaitForSingleObjectEx(hSemaphore, INFINITE, FALSE);
#endif
#if ( OS_TYPE == OS_LINUX )
	struct semaphore *pSemaphore = (struct semaphore *)hSemaphore;
	if (NULL == hSemaphore)
		return;
	down(pSemaphore);
#endif
}

/********************************************************************/
GLOBAL VOID	OSSemaphoreSignal(		IN	HANDLE hSemaphore)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	Signal_Semaphore((UINT32)hSemaphore);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	ReleaseSemaphore(hSemaphore, 1, NULL);
#endif
#if ( OS_TYPE == OS_LINUX )
	struct semaphore *pSemaphore = (struct semaphore *)hSemaphore;
	if (NULL == hSemaphore)
		return;
	up(pSemaphore);
#endif
}

/* Critical Section (Mutex) */
/********************************************************************/
#if ( OS_TYPE == OS_LINUX )
typedef struct CRIT {
	unsigned long int saved_flags;
	BOOL valid;
	struct CRIT *prev;
} CRIT_T;
#endif

GLOBAL HANDLE __OSCriticalSectionCreate( char *file, int line )
{
#if ( OS_TYPE == OS_LINUX )
    CRIT_T *mutex  = kmalloc(sizeof(CRIT_T), GFP_KERNEL);
	mutex->saved_flags = 0;
	mutex->valid = FALSE;
	mutex->prev = NULL;
//	printk( "From file %s, line %d: got mutex 0x%lx\n", file, line, mutex );
    return ((HANDLE)mutex);
#endif
#if ( OS_TYPE == OS_WIN_9X )
#if ( UK_MODE == UK_KERNEL )
	return( (HANDLE)_CreateMutex(0/* boost */, 0/* flags*/));
#else
	HANDLE RetVal;
	RetVal = CreateMutex(NULL, TRUE, NULL);
	ReleaseMutex(RetVal); // initialize mutex to be unowned
	return RetVal;
#endif
#else
	return NULL;
#endif
}

/********************************************************************/
GLOBAL VOID	OSCriticalSectionDestroy(	IN	HANDLE hMutex)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	_DestroyMutex((UINT32)hMutex);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	CloseHandle(hMutex);
#endif
#if ( OS_TYPE == OS_LINUX )
	if ( NULL != hMutex )
	{
		kfree((CRIT_T *)hMutex);
	}
#endif
}

#if ( OS_TYPE == OS_LINUX )
#include "cpucnt.inl"
static DWORD cs_start;
/*  static int cs_count=0; */
static CRIT_T *curr_mutex=NULL;
#endif

/********************************************************************/
GLOBAL VOID	OSCriticalSectionAcquire(	IN	HANDLE hMutex)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	_EnterMutex((UINT32)hMutex, BLOCK_SVC_INTS);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	WaitForSingleObjectEx(hMutex, INFINITE, FALSE);
#endif
#if ( OS_TYPE == OS_LINUX )
	CRIT_T *mutex = (CRIT_T *) hMutex;
	unsigned long int old_fl = mutex->saved_flags;

	//printk("Entering CriticalSection %x\n",hMutex);
	save_flags( mutex->saved_flags );
	cli();
	if ( mutex->valid )
	{
		printk( "OSCriticalSectionAcquire: mutex 0x%08lx in use (0x%08lx)\n",
				(UINT32) hMutex, (UINT32) old_fl );
	}
	mutex->valid = TRUE;
	mutex->prev = curr_mutex;
	curr_mutex = mutex;
	cs_start = ReadCpuCnt();
/*  	if ( ++cs_count != 1 ) */
/*  	{ */
/*  		printk( "OSCriticalSectionAcquire: count is %d!!!\n", cs_count ); */
/*  	} */
#endif
}

/********************************************************************/
GLOBAL VOID	OSCriticalSectionRelease(	IN	HANDLE hMutex)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	_LeaveMutex((UINT32)hMutex);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	ReleaseMutex(hMutex);
#endif
#if ( OS_TYPE == OS_LINUX )
	CRIT_T *mutex = (CRIT_T *) hMutex;
	DWORD cs_end, tmp;
/*  	if ( --cs_count != 0 ) */
/*  	{ */
/*  		printk( "OSCriticalSectionRelease: count is %d!!!\n", cs_count ); */
/*  	} */
	tmp = cs_end = ReadCpuCnt();
	if ( curr_mutex != mutex )
	{
		printk("OSCriticalSectionRelease: acquired 0x%08lx, release 0x%08lx\n",
			   (UINT32) curr_mutex, (UINT32) mutex );
	}
	if ( !mutex->valid )
	{
		printk( "OSCriticalSectionRelease: mutex 0x%08lx invalid (0x%08lx)\n",
				(UINT32) hMutex, (UINT32) mutex->saved_flags );
	}
	curr_mutex = mutex->prev;
	mutex->valid = FALSE;
	cs_end -= cs_start;
	restore_flags( mutex->saved_flags );
	if ( cs_end > 2660000 ) /* More than 10mSec */
	{
		printk( "CriticalSection too long (%ld-%ld=%ld)\n",
				tmp, cs_start, cs_end );
	}
	//printk("Left CriticalSection %x\n",hMutex);
#endif
}

/********************************************************************/
GLOBAL	VOID	OSSetTimeSensitivity(	IN	UINT32				Interval)			
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	VTD_Begin_Min_Int_Period(Interval);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	timeBeginPeriod(Interval);
#endif
}
/********************************************************************/
GLOBAL	VOID	OSRestoreTimeSensitivity(		IN	UINT32				Interval)
{
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
	VTD_End_Min_Int_Period(Interval);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_USER) )
	timeEndPeriod(Interval);
#endif
}

/********************************************************************/
GLOBAL	HANDLE	OSDisableInterrupt(void)
{
	UINT16 wPreviousState;
#if ( COMPILER == CMP_MSDEV)
	__asm
    {
	    push  eax;
	    pushf;
	    pop   ax;
	    mov   wPreviousState, ax;
	    pop   eax;
	    cli;
    }
#else
    asm ( "\n\t"
	  "pushl %%eax \n\t"
	  "pushf \n\t"
	  "popw  %%ax \n\t"
	  "movw  %%ax, %0 \n\t"
	  "popl  %%eax \n\t"
	  "cli"

	  : "=m" (wPreviousState) );
#endif
	return( (HANDLE) (UINT) wPreviousState);
}

/********************************************************************/
GLOBAL	VOID	OSRestoreInterrupts(	IN	HANDLE hStatus)
{
 	UINT16 wPreviousState = (UINT16) (UINT) hStatus;
#if ( COMPILER == CMP_MSDEV)
	__asm
    {
	    push  eax;
	    mov   ax, wPreviousState;
	    test  ah, 0x02;
	    pop	eax;
	    jz    SHORT No_Enable;
	    sti;
	No_Enable:
	}
#else
   asm( "\n\t"
	"pushl  %%eax \n\t"
	"movw   %0, %%ax \n\t"
	"testb  $2, %%ah \n\t"
	"popl   %%eax \n\t"
	"jz     No_Enable \n\t"
	"sti \n"
        "No_Enable:\n"

	: : "m" (wPreviousState) );
#endif
}

/********************************************************************/
 
#if ( OS_TYPE == OS_LINUX )
STATIC	char FSaveMem[200];
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) ) 
STATIC REGSTAT	RegStat;
STATIC int	CurrRequest = NO_REQUESTS;
#endif
/**********************************************************************************
Function name :	OSFloatPrefix
Parameters :	None
Return value :	None
functionality :	Treats the FPU in order to enable float operations.
**********************************************************************************/
/**********************************************************************************/
#if ( OS_TYPE != OS_LINUX )
VOID InterruptEnable( INTERUPT_STATUS_STATUS Status )
{
 
   static WORD wEnterCount = 0;
   static WORD wPreviousState = 0;
	switch (Status)
   {
      case INTERUPT_STATUS_DISABLE:
         if (wEnterCount == 0)
		 {
	      __asm
            {
		  push  eax;
		  pushf;
		  pop   ax;
		  mov   wPreviousState, ax;
		  pop   eax;
		  cli;
            }
         }
		 else {
			 // Issue a warning, we are entering nested 'disable interrupts'
			 DPRINTF(("::CriticalSection (Enter) - Nested Disable interrupts"));
		 }
         break ;
      case INTERUPT_STATUS_RESTORE:
         if (wEnterCount != 0)
         {
            wEnterCount-- ;
            if (wEnterCount == 0)
            {
               __asm
               {
				  push  eax
                  mov   ax, wPreviousState
                  test  ah, 0x02
				  pop	eax
                  jz    SHORT No_Enable
                  sti
               No_Enable:
               }
            }
         }
		 if (wEnterCount > 0) {
			 // Issue a warning, we are in nested 'disable interrupts'
			 DPRINTF(("::CriticalSection (Exit) - Nested Disable interrupts"));
		 }
         break ;
   }
} 
#endif 
/**********************************************************************************/
 
int save_cr0(void);
void restore_cr0(int cr0);
 
GLOBAL	int OSFloatPrefix(void)
{
#if ( OS_TYPE == OS_LINUX )
    char tempbuffer[200];  
	unsigned long flags;
	int cr0;
	save_flags(flags);
	cli();
	cr0 = save_cr0();
	ClearTS();
	asm( "fsave (%0) \n\t"
		 "fwait \n"
		 : : "r" (tempbuffer) : "cc", "memory" );
    memcpy(FSaveMem,tempbuffer,200);
	restore_flags(flags);
	return((int)cr0);
#endif
 
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) ) 
	UINT8	TmpSave87State[120];
	UINT32	TmpSavedCR0;
//	HANDLE	hCriticalSection;

#if (OS_TYPE == OS_WIN_NT) || defined AMCHAL
	KIRQL CurIrql = KeGetCurrentIrql();
	if (CurIrql == PASSIVE_LEVEL) {
		return UNHANDLED_REQUEST;
	}
#endif

#if (OS_TYPE != OS_WIN_NT) && !defined AMCHAL
//	hCriticalSection = OSCriticalSectionCreate();
//	OSCriticalSectionAcquire(hCriticalSection );
	InterruptEnable(INTERUPT_STATUS_DISABLE);
#endif

	if (CurrRequest != NO_REQUESTS) {
#if (OS_TYPE != OS_WIN_NT) && !defined AMCHAL
//	OSCriticalSectionRelease(hCriticalSection );
//	OSCriticalSectionDestroy(hCriticalSection );
		InterruptEnable(INTERUPT_STATUS_RESTORE);
#endif
		return UNHANDLED_REQUEST;
	}

	CurrRequest = HANDLED_REQUEST;

    __asm mov eax,CR0
    __asm mov TmpSavedCR0,eax
    __asm clts

#if (OS_TYPE == OS_WIN_NT) || defined AMCHAL
	ClearEM();
#endif
	
	//__asm fstenv TmpSave87Env
	__asm fsave TmpSave87State
	__asm fwait

#if (OS_TYPE != OS_WIN_NT) && !defined AMCHAL
//	OSCriticalSectionRelease(hCriticalSection );
//	OSCriticalSectionDestroy(hCriticalSection );
	InterruptEnable(INTERUPT_STATUS_RESTORE);
#endif
	RegStat.SavedCR0 = TmpSavedCR0;
	memcpy( RegStat.Save87State, TmpSave87State, 120 );

#endif /* OS_VXD */
 
	return HANDLED_REQUEST;
}

/**********************************************************************************
Function name :	OSFloatPrefix
Parameters :	None
Return value :	None
functionality :	Restores the FPU after float operations.
**********************************************************************************/
#if !( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
#if ( COMPILER == CMP_MSDEV)
#pragma warning (disable:4100)
#endif
#endif
GLOBAL	BOOL OSFloatSuffix(int Id)
{
#if ( OS_TYPE == OS_LINUX )
    char tempbuffer[200];  
	unsigned long flags;
	save_flags(flags);
	cli();
    memcpy(tempbuffer,FSaveMem,200);
	ClearTS();
	asm( "frstor (%0) \n\t"
		 "fwait \n"
		 : : "r" (tempbuffer) : "cc", "memory" );
	restore_cr0(Id);
    restore_flags(flags);
#endif
#if ( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
//	HANDLE	hCriticalSection;
	UINT8	TmpSave87State[120];
	UINT32	TmpSavedCR0 = RegStat.SavedCR0;
	if ((Id != HANDLED_REQUEST) || (CurrRequest != HANDLED_REQUEST)) {
		return FALSE;
	}
	

	memcpy( TmpSave87State, RegStat.Save87State, 120 );

#if (OS_TYPE != OS_WIN_NT) && !defined AMCHAL
//	hCriticalSection = OSCriticalSectionCreate();
//	OSCriticalSectionAcquire(hCriticalSection );
	InterruptEnable(INTERUPT_STATUS_DISABLE);
#endif
    __asm clts
	__asm frstor TmpSave87State
	__asm fwait
 
    __asm mov eax, TmpSavedCR0
    __asm mov CR0, eax
 
	CurrRequest = NO_REQUESTS;
#if (OS_TYPE != OS_WIN_NT) && !defined AMCHAL
//	OSCriticalSectionRelease(hCriticalSection );
//	OSCriticalSectionDestroy(hCriticalSection );
	InterruptEnable(INTERUPT_STATUS_RESTORE);
#endif
#endif /* OS_VXD */
	return TRUE;
}
#if !( (OS_TYPE == OS_WIN_9X) && (UK_MODE == UK_KERNEL) )
#if ( COMPILER == CMP_MSDEV)
#pragma warning (default:4100)
#endif
#endif

/********************************************************************/
#if (OS_TYPE == OS_LINUX)

HANDLE OSCreateTimer(UINT32 msec, PVOID pCBFunc, PVOID pRefData)
{
	struct timer_list*	pTimer;

	if ( pCBFunc == NULL )
	{
		return (NULL);
	}

	pTimer = kmalloc(sizeof(struct timer_list),GFP_KERNEL);
	
	if ( NULL == pTimer )
	{
		printk ("OSCreateTimer:: failed to allocate pTimer\n");
		return (NULL);
	}

	memset(pTimer,0,sizeof(struct timer_list));

	init_timer(pTimer);
	pTimer->function = pCBFunc ;
	pTimer->data = (unsigned long)pRefData;
	pTimer->expires = jiffies + msec/10;
	return ((HANDLE)pTimer);
}

void OSSetTimer(PVOID pTimer)
{
	if ( pTimer == NULL )
	{
		return;
	}
	add_timer(pTimer);
}

void OSCancelTimer(PVOID pTimer)
{
	if ( pTimer == NULL )
	{
		return;
	}
	del_timer(pTimer);
}

void OSChangeTimerTimeOut(PVOID pTimer, UINT32 msec)
{
	struct timer_list*	pTimerH = (struct timer_list*)pTimer;
	if ( pTimer == NULL )
	{
		return;
	}
	pTimerH->expires = jiffies + msec/10;
}

void OSDestroyTimer(PVOID pTimer)
{
	if ( pTimer == NULL )
	{
		return;
	}

	del_timer(pTimer);
	kfree(pTimer);
}
#endif

/********************************************************************/

#if ( OS_TYPE == OS_LINUX )

void ClearTS (void)
{
   asm( "movl   %cr0, %eax \n\t"
		"andl   $0xfffffff7, %eax \n\t"
		"movl   %eax, %cr0 \n" );
}
 
int save_cr0(void)
{
	int cr0;
	
	asm( "movl %%cr0, %%eax" : "=a" (cr0) : : "cc" );
 
	return cr0;
}
 
void restore_cr0(int cr0)
{
	asm( "movl  %%eax, %%cr0" : "=a" (cr0) : : "cc" );
}

#endif
