/*
 * ktime.c - implement some time functions for the module.
 */

#define EXPORT_SYMTAB
#define __NO_VERSION__
#include <linux/module.h>

#include "ktime.h"


char* ktime_month_name[12] = {
    "January", "February", "March", "April", "May", "June",
    "July", "August", "September", "October", "November", "December",
};

char* ktime_month_sname[12] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
};

EXPORT_SYMBOL_NOVERS(ktime_month_name);
EXPORT_SYMBOL_NOVERS(ktime_month_sname);
EXPORT_SYMBOL_NOVERS(ktime_tv_sec_to_tm);

static int month_len[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

void ktime_tv_sec_to_tm( struct tm *ptm, time_t time )
{
    int i, days_from_1970;

    days_from_1970 = time / (24 * 3600);
    time -= days_from_1970 * 24 * 3600;

    ptm->tm_hour = time / 3600;
    time -= ptm->tm_hour * 3600;

    ptm->tm_min = time / 60;
    time -= ptm->tm_min * 60;

    ptm->tm_sec = time;

    ptm->tm_year = (4 * days_from_1970 + 2) / 1461; /* 365.25 days per year */
    days_from_1970 -= (1461 * ptm->tm_year + 1) / 4;
    ptm->tm_year += 70;
    ptm->tm_yday = days_from_1970;

    if ( (ptm->tm_year & 0x3) == 0 )
    {
	month_len[1] = 29; /* February ... */
    }
    i = 0;
    while ( days_from_1970 >= month_len[i] )
    {
	days_from_1970 -= month_len[i++];
    }
    ptm->tm_mon = i;
    ptm->tm_mday = days_from_1970+1;

    month_len[1] = 28; /* February ... */

}  /* tv_sec_to_tm */
