/*
 * linux_dbgscr.c - implements the linux actual dbgscr writing to file.
 *                  supports both user and kernel mode.
 */

#define EXPORT_SYMTAB
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include "ktime.h"
#include "ossysenv.h"
#include "linux_dbgscr.h"
#include "scrcommon.h"

#define MOD_NAME "DbgScr"
#define MAX_FILE_SIZE 1024*500
 
EXPORT_SYMBOL_NOVERS(LinuxScrSetEntryPointCallback);
EXPORT_SYMBOL_NOVERS(LinuxScrWakeup);
EXPORT_SYMBOL_NOVERS(LinuxScrSleep);

/* static int flush_mode = 0; */
/* MODULE_PARM( flush_mode, "i" ); */

typedef enum {
	SCR_TQ_OPEN_FILE,
	SCR_TQ_WRITE,
	SCR_TQ_CLOSE_FILE,
	SCR_TQ_LAST,
} ScrTQop_T;
 
 
static char *scr_file_name = NULL;
MODULE_PARM( scr_file_name, "s" );

#define MAX_SCR_FILE_NAME 256

static BOOL scr_active = FALSE;
static struct file *scr_file = NULL;
static char *print_buff = NULL;
static DWORD file_offset = 0;
static BOOL incrementable_file_name = FALSE;
static char scr_file_name_format[MAX_SCR_FILE_NAME] = { '\0' };
static char scr_file_tmp_name[MAX_SCR_FILE_NAME] = { '\0' };
static struct tq_struct ScrTaskQueue[3];
static BOOL ScrTQinUse[SCR_TQ_LAST] = { FALSE, FALSE, FALSE };

 
static VOID ScrTQOpenFile( PVOID arg );
/* static VOID ScrTQWrite( PVOID arg ); */
static VOID ScrTQCloseFile( PVOID arg );
static BOOL ScrInitGetFileName( void );
static BOOL ScrGetNextFileName( char *next_name, int index );
static VOID __fastcall KernelPrintFunc( PCHAR str );


int init_module( void )
{

#if 0
    struct timeval tmv;
    struct tm stm;

    get_fast_time( &tmv );
    ktime_tv_sec_to_tm( &stm, tmv.tv_sec );
    printk( "-=-=-=-=- V34 up at: %02d/%02d/%d, %02d:%02d:%02d -=-=-=-=-\n",
			stm.tm_mday, stm.tm_mon+1, 1900+stm.tm_year,
			stm.tm_hour, stm.tm_min, stm.tm_sec );
    printk( MOD_NAME ": init_module!!\n");
#endif

    printk( "-=-=-=-=- " MOD_NAME " compiled on %s %s -=-=-=-=-\n",
			__DATE__, __TIME__ );

	if ( TRUE == ScrInitGetFileName() )
	{
		print_buff = vmalloc( MAX_FILE_SIZE );
		if ( NULL == print_buff )
		{
			printk( MOD_NAME ": Could not alloc 500K for print buffer. "
					"Stopping SCR.\n" );
		}
		else
		{
			scr_active = TRUE;
		}
	}
	else
		printk( MOD_NAME ": not active\n" );

    printk( MOD_NAME ": loaded.\n" );

    return 0;

}  /* init_module */


void cleanup_module( void )
{

    printk( MOD_NAME ": cleanup_module!!\n");

	if ( NULL != print_buff )
	{
		vfree( print_buff );
		print_buff = NULL;
	}

    printk( MOD_NAME ": unloaded.\n" );

}  /* cleanup_module */


BOOL LinuxScrSetEntryPointCallback( DWORD epcbk )
{

	if ( !scr_active /* || (NULL == scr_file) */ )
	{
		((PCALLBACK) epcbk)( (PVOID) NULL );
		return FALSE;
	}

	((PCALLBACK) epcbk)( (PVOID) KernelPrintFunc );

	return TRUE;

}  /* LinuxScrSetEntryPointCallback */


static BOOL ScrInitGetFileName( void )
{
	int i, X_num;

	if ( NULL == scr_file_name )
	{
		return FALSE;
	}

	for ( i = 0; scr_file_name[i] != '\0'; i++ )
	{
		if ( scr_file_name[i] == 'X' )
		{
			break;
		}
		scr_file_name_format[i] = scr_file_name[i];
	}

	if ( scr_file_name[i] == '\0' )
	{
		scr_file_name_format[i] = '\0';
		printk( MOD_NAME ": scr_file_name_format is %s (not incrementable)\n",
				scr_file_name_format );
		return TRUE;
	}

	X_num = 1;
	for ( i++; scr_file_name[i] == 'X'; i++ )
		X_num++;

	if ( X_num > 2 )
	{
		printk( MOD_NAME ": found incrementable format (%d digits)\n", X_num );
		sprintf( &scr_file_name_format[i-X_num], "%%0%dd%s",
				 X_num, &scr_file_name[i] );
		incrementable_file_name = TRUE;
	}
	else
	{
		sprintf( &scr_file_name_format[i-X_num], "%s%s",
				 (X_num == 1) ? "X" : "XX", &scr_file_name[i] );
	}

	printk( MOD_NAME ": scr_file_name_format is `%s'\n",scr_file_name_format );

	return TRUE;

}  /* ScrInitGetFileName */


static VOID ScrTQOpenFile( PVOID arg )
{
	static int file_index = 0;
	mm_segment_t fs;
    struct timeval tmv;
    struct tm stm;

//	printk("SANITY FOR DEBUG :::::::::: ScrTQOpenFile ");
	if ( (FALSE == scr_active) || (NULL != scr_file) ||
		 (FALSE == ScrTQinUse[SCR_TQ_OPEN_FILE]) ||
		 (SCR_TQ_OPEN_FILE != ((ScrTQop_T) arg)) )
	{
		printk( MOD_NAME ": ScrTQOpenFile, scr_active=%d, scr_file=0x%x, "
				"ScrTQinUse[%d]=%d, arg=%d. Exiting.\n",
				scr_active, (unsigned int) scr_file,
				SCR_TQ_OPEN_FILE, ScrTQinUse[SCR_TQ_OPEN_FILE], (int) arg );
		ScrTQinUse[SCR_TQ_OPEN_FILE] = FALSE;
		return;
	}

	if ( TRUE == ScrGetNextFileName(scr_file_tmp_name, ++file_index) )
	{
		fs = get_fs();
		set_fs( KERNEL_DS );

		printk( MOD_NAME ": scr file name is `%s'"/*, flush mode %s*/".\n",
				scr_file_tmp_name/* , flush_mode ? "on" : "off" */ );

		scr_file = filp_open( scr_file_tmp_name, O_CREAT | 2 | O_TRUNC, 0777 );
		if ( (((int) scr_file) & 0xf0000000) == 0xf0000000 )
		{
			printk( MOD_NAME ": Could not open %s (error=%d). SCR disabled.\n",
					scr_file_tmp_name, -((int) scr_file) );
			scr_file = NULL;
			set_fs( fs );
			ScrTQinUse[SCR_TQ_OPEN_FILE] = FALSE;
			return;
		}
//		printk( MOD_NAME ": scr_file is 0x%x\n", (unsigned int) scr_file );

		if ( NULL == scr_file->f_op->write )
		{
			printk( MOD_NAME ": File %s does not have a write function!!! "
					"Stopping SCR.\n", scr_file_tmp_name );
			filp_close( scr_file, NULL );
			scr_file = NULL;
			set_fs( fs );
			ScrTQinUse[SCR_TQ_OPEN_FILE] = FALSE;
			return;
		}

		get_fast_time( &tmv );
		ktime_tv_sec_to_tm( &stm, tmv.tv_sec );
		sprintf( print_buff,
				 "ALL Run start: %02d/%02d/%04d %02d:%02d:%02d\n"
				 "ALL -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-\n",
				 stm.tm_mday, stm.tm_mon+1, 1900+stm.tm_year,
				 stm.tm_hour, stm.tm_min, stm.tm_sec );
		scr_file->f_op->write( scr_file, print_buff,
							   strlen(print_buff), &scr_file->f_pos );
		print_buff[0] = '\0';
		file_offset = 0;

		set_fs( fs );
		ScrTQinUse[SCR_TQ_OPEN_FILE] = FALSE;
		return;
	}
	else
	{
		printk( MOD_NAME ": wakeup: ScrGetNextName failed. Stopping SCR.\n" );
		scr_active = FALSE;
	}

	set_fs( fs );
	ScrTQinUse[SCR_TQ_OPEN_FILE] = FALSE;

}  /* ScrTQOpenFile */


#if 0
static VOID ScrTQWrite( PVOID arg )
{
	

}  /* ScrTQWrite */
#endif


static VOID ScrTQCloseFile( PVOID arg )
{
	int close_ret;
	DWORD nw;
	DWORD write_count;
	mm_segment_t fs;
 
 
	if ( (FALSE == scr_active) || (NULL == scr_file) ||
		 (FALSE == ScrTQinUse[SCR_TQ_CLOSE_FILE]) ||
		 (SCR_TQ_CLOSE_FILE != ((ScrTQop_T) arg)) )
	{
		printk( MOD_NAME ": ScrTQCloseFile, scr_active=%d, scr_file=0x%x, "
				"ScrTQinUse[%d]=%d, arg=%d. Exiting.\n",
				scr_active, (unsigned int) scr_file,
				SCR_TQ_CLOSE_FILE, ScrTQinUse[SCR_TQ_CLOSE_FILE], (int) arg );
		ScrTQinUse[SCR_TQ_CLOSE_FILE] = FALSE;
		return;
	}

 
	fs = get_fs();
	set_fs( KERNEL_DS );
	write_count =  file_offset + 1;
	nw =  scr_file->f_op->write(scr_file, print_buff, write_count , &scr_file->f_pos) ;
	if ( nw != write_count )
	{
		printk( MOD_NAME ": ========== wrote only %ld of %ld\n to SCR file  ===========", nw, write_count);
	}
 
 
	close_ret = filp_close( scr_file, NULL );
	if ( 0 != close_ret )
	{
		printk( MOD_NAME ": Close of SCR file failed (error=%d)\n",
				close_ret );
	}
	printk( MOD_NAME ": scr file `%s' closed.\n", scr_file_tmp_name );

	scr_file = NULL;
	ScrTQinUse[SCR_TQ_CLOSE_FILE] = FALSE;

}  /* ScrTQCloseFile */


static BOOL ScrGetNextFileName( char *next_name, int index )
{
	if ( scr_file_name_format[0] == '\0' )
	{
		return FALSE;
	}

	if ( TRUE == incrementable_file_name )
	{
		sprintf( next_name, scr_file_name_format, index );
	}
	else {
		sprintf( next_name, scr_file_name_format );
	}

	return TRUE;

}  /* ScrGetNextFileName */


BOOL LinuxScrWakeup( VOID )
{
	printk("LinuxScrWakeup =======================\n" );

	if ( FALSE == scr_active )
		return FALSE;

	if ( (FALSE == scr_active) || (NULL != scr_file) ||
		 (TRUE == ScrTQinUse[SCR_TQ_OPEN_FILE]) )
	{
		printk( MOD_NAME ": LinuxScrWakeup, scr_active=%d, scr_file=0x%x, "
				"ScrTQinUse[%d]=%d. Not scheduling.\n",
				scr_active, (unsigned int) scr_file,
				SCR_TQ_OPEN_FILE, ScrTQinUse[SCR_TQ_OPEN_FILE] );
		return FALSE;
	}

	ScrTaskQueue[SCR_TQ_OPEN_FILE].routine = ScrTQOpenFile;
	ScrTaskQueue[SCR_TQ_OPEN_FILE].data = (PVOID) SCR_TQ_OPEN_FILE;
	ScrTQinUse[SCR_TQ_OPEN_FILE] = TRUE;
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )
	queue_task( &ScrTaskQueue[SCR_TQ_OPEN_FILE], &tq_scheduler );
#else
	schedule_task(&ScrTaskQueue[SCR_TQ_OPEN_FILE]);
#endif

	return TRUE;

}  /* LinuxScrWakeup */


static VOID __fastcall KernelPrintFunc( PCHAR str )
{
	int str_len;
 
	mm_segment_t fs;

	if ( !scr_active || (NULL == scr_file) )
		return;

	fs = get_fs();
	set_fs( KERNEL_DS );

	str_len = strlen(str);
	if ( file_offset + str_len + 1 < MAX_FILE_SIZE )
	{
		strcpy( print_buff + file_offset, str );
		file_offset += str_len;
		print_buff[file_offset] = '\n';
		file_offset++;
		print_buff[file_offset] = '\0';
	}
	else
	{
		printk( MOD_NAME ": Buffer full - stop writing to buffer .\n" );
	}
 

	set_fs( fs );

}  /* KernelPrintFunc */


BOOL LinuxScrSleep( VOID )
{

	if ( FALSE == scr_active )
		return FALSE;

	if ( (FALSE == scr_active) || (NULL == scr_file) ||
		 (TRUE == ScrTQinUse[SCR_TQ_CLOSE_FILE]) )
	{
		printk( MOD_NAME ": LinuxScrSleep, scr_active=%d, scr_file=0x%x, "
				"ScrTQinUse[%d]=%d. Not scheduling.\n",
				scr_active, (unsigned int) scr_file,
				SCR_TQ_CLOSE_FILE, ScrTQinUse[SCR_TQ_CLOSE_FILE] );
		return FALSE;
	}

	ScrTaskQueue[SCR_TQ_CLOSE_FILE].routine = ScrTQCloseFile;
	ScrTaskQueue[SCR_TQ_CLOSE_FILE].data = (PVOID) SCR_TQ_CLOSE_FILE;
	ScrTQinUse[SCR_TQ_CLOSE_FILE] = TRUE;
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )
	queue_task( &ScrTaskQueue[SCR_TQ_CLOSE_FILE], &tq_scheduler );
#else
	schedule_task(&ScrTaskQueue[SCR_TQ_CLOSE_FILE]);
#endif

	return TRUE;

}  /* LinuxScrSleep */
