# Make file for the modulations libraries - mmx_code, clib and dsp_modules.

ifeq ($(MLOUTDIR),)
error "You must define MLOUTDIR to where the modulations lib must be compiled"
endif

OCT_OUTDIR = $(MLOUTDIR)/oct_files
include $(OCTOPUS_DIR)/linux/octopus_export.mak

MOD_LIBS = $(DSPM_LIB) $(MMX_CODE_LIB) $(CLIB_LIB) $(LINUX_LIB) $(OCTC_LIB)

# Rules for mmx_code:
#---------------------

MMX_CODE_FL  = mmxadaptccc mmxadaptrll mmxcadapt mmxcladapt mmxen64ccl 
MMX_CODE_FL += mmxen8nrrl mmxfactor16bit mmxfadeadaptrll mmxfir mmxfir32h16 
MMX_CODE_FL += mmxfirc mmxfirccc mmxfirccc1 mmxfircl mmxfirqf mmxfirrcc2 
MMX_CODE_FL += mmxfirrll mmxfirrlr mmxfirrrl mmxladapt mmxvmul mmxvmul16 
MMX_CODE_FL += mmxvmulccr mmxvmulrecl retrain vit32 viterbi_f 
MMX_CODE_FILES = $(MMX_CODE_FL:%=$(SW_MODEM_DIR)/mmx_code/%)

MMX_CODE_ODIR = $(MLOUTDIR)/mmx_code
ALL_ODIRS += $(MMX_CODE_ODIR)

MMX_CODE_OBJS = $(patsubst %,$(MMX_CODE_ODIR)/%.o,$(notdir $(MMX_CODE_FILES)))

MMX_CODE_LIB = $(MMX_CODE_ODIR)/libmmx_code.a

vpath %.asm $(SW_MODEM_DIR)/mmx_code

$(MMX_CODE_ODIR)/%.o: %.asm
	$(ASM_CVT) $< $(@:%.o=%.s)
	$(ASM) $(ASM_FLAGS) -o $@ $(@:%.o=%.s)

$(MMX_CODE_LIB): $(MMX_CODE_OBJS) 
	ar -r $(MMX_CODE_LIB) $(MMX_CODE_OBJS)


# Rules for clib:
#-----------------

CLIB_FILES  = $(SW_MODEM_DIR)/globals/appglobeng.cpp 
CLIB_FILES += $(SW_MODEM_DIR)/globals/globeng.cpp 
CLIB_FILES += $(SW_MODEM_DIR)/dspmodls/memmgr.cpp 

CLIB_ODIR = $(MLOUTDIR)/clib
ALL_ODIRS += $(CLIB_ODIR)

CLIB_OBJS = $(patsubst %.cpp,$(CLIB_ODIR)/%.o,$(notdir $(CLIB_FILES)))

CLIB_LIB = $(CLIB_ODIR)/libclib.a

vpath %.cpp $(SW_MODEM_DIR)/globals $(SW_MODEM_DIR)/dspmodls 

$(CLIB_ODIR)/%.o: %.cpp
	$(GPP) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GPP_TEMPS_EXT),$(notdir $@)) $(CLIB_ODIR)

$(CLIB_LIB): $(CLIB_OBJS) 
	ar -r $(CLIB_LIB) $(CLIB_OBJS)


# Rules for linux_lib:
#----------------------

LINUXLIB_FILES  = $(SW_MODEM_DIR)/linux/ktime 

LINUXLIB_ODIR = $(MLOUTDIR)/linux
ALL_ODIRS += $(LINUXLIB_ODIR)

LINUXLIB_OBJS = $(patsubst %,$(LINUXLIB_ODIR)/%.o,$(notdir $(LINUXLIB_FILES)))

LINUX_LIB = $(LINUXLIB_ODIR)/liblinux.a

vpath %.c $(SW_MODEM_DIR)/linux 
vpath %.cpp $(SW_MODEM_DIR)/linux 

$(LINUXLIB_ODIR)/%.o: %.c
	$(GCC) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GCC_TEMPS_EXT),$(notdir $@)) $(LINUXLIB_ODIR)

$(LINUXLIB_ODIR)/%.o: %.cpp
	$(GPP) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GPP_TEMPS_EXT),$(notdir $@)) $(LINUXLIB_ODIR)

$(LINUX_LIB): $(LINUXLIB_OBJS) 
	ar -r $(LINUX_LIB) $(LINUXLIB_OBJS)


# Rules for dsp_modules and genmodem:
#-----------------------------------
DSPMSF  = asm_globals.cpp bitlayer.cpp bits.cpp bitstreamfliplr.cpp ccor.cpp 
DSPMSF += carr_est.cpp coarseplls.cpp codecgain.cpp dbgfile.cpp dbgscr.cpp 
DSPMSF += dbgwatch.cpp descrambler.cpp ecsm.cpp equalizer.cpp fec.cpp fir.cpp 
DSPMSF += fredmachine.cpp g711.cpp globals.cpp goddard.cpp input_dec.cpp 
DSPMSF += mathfun.cpp msg.cpp nsp.cpp nsp_vect.cpp patterndet.cpp pll.cpp 
DSPMSF += poweriir.cpp rtools.cpp scrambler.cpp sdetect.cpp slicexsostive.cpp 
DSPMSF += srrcgenerator.cpp ssbardet.cpp statemachine.cpp symagc.cpp 
DSPMSF += timeouts.cpp tone.cpp tonef.cpp tx.cpp pj.cpp agc.cpp
DSPMSG  = datapumpconfig.cpp modem.cpp 
DSPMSS  = numconv.cpp stringconv.cpp datapumpstatus.cpp

DSPM_FILES  = $(addprefix $(SW_MODEM_DIR)/dspmodls/, $(DSPMSF))
DSPM_FILES += $(addprefix $(SW_MODEM_DIR)/genmodem/, $(DSPMSG))
DSPM_FILES += $(addprefix $(SW_MODEM_DIR)/sw_modules/, $(DSPMSS))
DSPM_FILES += $(SW_MODEM_DIR)/tstpoint/tp.cpp

DSPM_ODIR = $(MLOUTDIR)/dsp_modules
ALL_ODIRS += $(DSPM_ODIR)

DSPM_OBJS = $(patsubst %.cpp,$(DSPM_ODIR)/%.o,$(notdir $(DSPM_FILES)))

DSPM_LIB = $(DSPM_ODIR)/libdspm.a

vpath %.cpp $(SW_MODEM_DIR)/dspmodls $(SW_MODEM_DIR)/genmodem
vpath %.cpp $(SW_MODEM_DIR)/sw_modules $(SW_MODEM_DIR)/tstpoint

$(DSPM_ODIR)/%.o: %.cpp
	$(GPP) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GPP_TEMPS_EXT),$(notdir $@)) $(DSPM_ODIR)

$(DSPM_LIB): $(DSPM_OBJS) 
	ar -r $(DSPM_LIB) $(DSPM_OBJS)

# DO NOT DELETE
