# makefile for the v34 module under linux

include $(MAK_ENV)

V34_ODIR = $(V34_DIR)/obj
V34CONMOD = $(V34_ODIR)/v34cmodule.o 
V34MODULE = $(V34_ODIR)/v34module.o 
ALL_MODULES += $(V34MODULE) #$(V34CONMOD) 

ifneq ($(MAIN_MAKE),yes)

# if called from within another makefile, do not override:
ifneq ($(INCLUDED_MAK),yes)

INCLUDES  = -I $(COMMON_DIR)/include		\
			-I $(OCTOPUS_DIR)/include		\
			-I $(OCTOPUS_DIR)/osspecific	\
			-I $(SW_MODEM_DIR)/include		\
			-I $(SW_MODEM_DIR)/dspmodls		\
			-I $(SW_MODEM_DIR)/linux		\
			-I $(SW_MODEM_DIR)/globals		\
			-I $(SW_MODEM_DIR)/genmodem		\
			-I $(SW_MODEM_DIR)/tstpoint		\
			-I $(SW_MODEM_DIR)/sw_modules

C_FLAGS =	$(INCLUDES)							\
			$(ALL_FLAGS)						\
			-DCREATE_V34_INSTANCE_FUNCTIONS=1

all : ALLODIRS v34bmod #v34bcon #devtst

endif # ifneq ($(INCLUDED_MAK))


# Other LIBRARIES (mmx_code, clib & dspm):
#------------------------------------------

MLOUTDIR = $(V34_ODIR)
include linux/modulation_libs.mak

# Rules for v34:
#----------------

V34_FL  = bitinv.cpp constel.cpp crc.cpp decoder.cpp dewarp.cpp encoder.cpp
V34_FL += fagc.cpp fdatr.cpp fequalizer.cpp fft320.cpp fpdfe.cpp fpll.cpp
V34_FL += frame_map.cpp info.cpp md_table.cpp mp.cpp nsp_fft16.cpp
V34_FL += nsp_fft32.cpp periodicseq.cpp phase2.cpp phase2errrecover.cpp
V34_FL += ppdetect.cpp precoder.cpp probing.cpp rx_dpsk.cpp rxv34.cpp
V34_FL += shell.cpp slicer.cpp tablesv34trn.cpp tx_dpsk.cpp txv34.cpp
V34_FL += txv34phase2.cpp txv34symbols.cpp v34.cpp v34config.cpp
V34_FL += v34interface.cpp v34params.cpp v34symrx.cpp v34symrxsm.cpp
V34_FL += viterbi16state.cpp viterbi32state.cpp viterbigeneric.cpp
V34_FL += viterbishell.cpp w320fft.cpp w96fft.cpp warp.cpp v34status.cpp
V34_FILES = $(V34_FL:%=$(SW_MODEM_DIR)/v34/%)

V34F_ODIR = $(V34_ODIR)/v34
ALL_ODIRS += $(V34F_ODIR)

V34_OBJS = $(patsubst %.cpp,$(V34F_ODIR)/%.o,$(notdir $(V34_FILES)))

V34_LIB = $(V34F_ODIR)/libv34.a

vpath %.cpp $(SW_MODEM_DIR)/v34 
vpath %.c $(OCTOPUS_DIR)/osspecific

$(V34F_ODIR)/%.o: %.cpp
	$(GPP) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GPP_TEMPS_EXT),$(notdir $@)) $(V34F_ODIR)

$(V34_LIB): $(V34_OBJS) 
	ar -r $(V34_LIB) $(V34_OBJS)


# Rules for v34_batch:
#----------------------

V34B_FL  = batchconfiguration.cpp bler.cpp imd.cpp modembatchrunner.cpp modembatchthread.cpp noise.cpp stdafx.cpp tester.cpp v34batch.cpp
# logue.c
V34B_FILES = $(V34B_FL:%=$(SW_MODEM_DIR)/modembatch/%)

V34B_ODIR = $(V34_ODIR)/batch
ALL_ODIRS += $(V34B_ODIR)

V34B_OBJS = $(patsubst %.cpp,$(V34B_ODIR)/%.o,$(notdir $(V34B_FILES)))

V34B_LIB = $(V34B_ODIR)/libv34b.a

vpath %.cpp $(SW_MODEM_DIR)/modembatch

$(V34B_ODIR)/%.o: %.cpp
	$(GPP) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GPP_TEMPS_EXT),$(notdir $@)) $(V34B_ODIR)

$(V34B_LIB): $(V34B_OBJS)
	ar -r $(V34B_LIB) $(V34B_OBJS)



# Rules for v34module:
#----------------------

V34M_FL  = v34wrapper v34device
V34M_FILES = $(V34M_FL:%=$(SW_MODEM_DIR)/linux/%)

V34M_ODIR = $(V34_ODIR)/v34module
ALL_ODIRS += $(V34M_ODIR)

V34M_OBJS = $(patsubst %,$(V34M_ODIR)/%.o,$(notdir $(V34M_FILES)))

V34M_LIB = $(V34M_ODIR)/libv34m.a

vpath %.c $(SW_MODEM_DIR)/linux
vpath %.cpp $(SW_MODEM_DIR)/linux 

$(V34M_ODIR)/%.o: %.c
	$(GCC) $(RAB) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GCC_TEMPS_EXT),$(notdir $@)) $(V34M_ODIR)

$(V34M_ODIR)/%.o: %.cpp
	$(GPP) $(RAB) $(C_FLAGS) -c -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GPP_TEMPS_EXT),$(notdir $@)) $(V34M_ODIR)

$(V34M_LIB): $(V34M_OBJS) 
	ar -r $(V34M_LIB) $(V34M_OBJS)

V34MODSRC = $(SW_MODEM_DIR)/linux/v34module.c
# When compiling to run as batch:
V34CONMAIN = $(V34M_ODIR)/v34conmain.o
v34bcon: RAB = -DRUN_AS_BATCH=1
$(V34CONMAIN): $(V34MODSRC)
	$(GCC) $(RAB) $(C_FLAGS) -c -o $@ $(V34MODSRC)
	@/bin/mv -f $(patsubst %.c,%$(GCC_TEMPS_EXT),$(notdir $<)) $(V34M_ODIR)

# When compiling to run as module:
V34MODMAIN = $(V34M_ODIR)/v34modmain.o
v34bmod: RAB = -DRUN_AS_BATCH=0
$(V34MODMAIN): $(V34MODSRC)
	$(GCC) $(RAB) $(C_FLAGS) -c -o $@ $(V34MODSRC)
	@/bin/mv -f $(patsubst %.c,%$(GCC_TEMPS_EXT),$(notdir $<)) $(V34M_ODIR)

#V34MOD_OBJS  = $(V34B_LIB) 
#V34MOD_OBJS +=  $(OCT_OUTDIR)/ostime.o 
V34MOD_OBJS += $(V34M_LIB) $(V34_LIB) 
V34MOD_OBJS += $(MOD_LIBS) $(DSPM_ODIR)/asm_globals.o 

v34bcon: $(V34CONMOD) 
v34bmod: $(V34MODULE) 

$(V34CONMOD): $(V34CONMAIN) $(V34MOD_OBJS)
	$(LD) -o $@ $(LD_FLAGS) -r $(V34CONMAIN) $(V34MOD_OBJS) 

$(V34MODULE): $(V34MODMAIN) $(V34MOD_OBJS)
	$(LD) -o $@ $(LD_FLAGS) -r $(V34MODMAIN) $(V34MOD_OBJS) 


# Rules for $(V34M_ODIR)/testdev:
#---------------------------------

devtst: $(V34M_ODIR)/testdev

$(V34M_ODIR)/testdev.o: linux/testdev.c
	$(GCC) $(C_FLAGS) -o $@ $<
	@/bin/mv -f $(patsubst %.o,%$(GCC_TEMPS_EXT),$(notdir $@)) $(V34M_ODIR)

# if called from within another makefile, do not override:
ifneq ($(INCLUDED_MAK),yes)

# Output directories rule (must be after all ALL_ODIRS += statements:
ALLODIRS: $(ALL_ODIRS)

$(ALL_ODIRS):
	mkdir -p $@

endif # ifneq ($(INCLUDED_MAK))

endif # of if (MAIN_MAKE)

# DO NOT DELETE
