/*
 * v34device.c - implement device related functionality.
 */

#define __NO_VERSION__
#include "ossysenv.h"
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include "v34device.h"
#include "v34module.h"
#include "v34wrapper.h"
#include "vmod_ioctl.h"


static int Vmod_open( struct inode *ip, struct file *fp );
static int Vmod_close( struct inode *ip, struct file *fp );
static ssize_t V34mod_read( struct file *fp,
			    char *b, size_t nb, loff_t *fo );
static ssize_t V34mod_write( struct file *fp,
			     const char *b, size_t nb, loff_t *fo );
static int V34mod_ioctl( struct inode *ip, struct file *fp,
			 unsigned int cmd, unsigned long arg );

static struct file_operations Vmod_fops = {
    NULL,	/* function for `llseek' file-op */
    NULL,	/* function for `read' file-op */	/* v??mod_read */
    NULL,	/* function for `write' file-op */	/* v??mod_write */
    NULL,	/* function for `readdir' file-op */
    NULL,	/* function for `poll' file-op */
    NULL,	/* function for `ioctl' file-op */	/* v??mod_ioctl */
    NULL,	/* function for `mmap' file-op */
    Vmod_open,	/* function for `open' file-op */
    NULL,	/* function for `flush' file-op */
    Vmod_close,	/* function for `release' file-op */
    NULL,	/* function for `fsync' file-op */
    NULL,	/* function for `fasync' file-op */
    NULL,	/* function for `check_media_change' file-op */
    NULL,	/* function for `revalidate' file-op */
    NULL,	/* function for `lock' file-op */
};

static int Vmod_major = 0;   /* Set to 0 for auto-major, or specific number. */

Vmodem_t Vmodems[2*NUM_VS];


BOOL device_init( void )
{
    int i, result;

    for ( i = 0; i < 2*NUM_VS; i++ )
    {
	Vmodems[i].modem = NULL;
	Vmodems[i].modem_ver = 0;
	Vmodems[i].globeng_ver = 0;
    }

    result = register_chrdev( Vmod_major, MOD_NAME, &Vmod_fops );
    if ( result < 0 )
    {
/*  	if ( Vmod_major != 0 ) */
/*  	    printk( MOD_NAME ": Can't get device major #%d\n", Vmod_major ); */
/*  	else */
/*  	    printkn( MOD_NAME ": Can't get device major #(dynamic)\n" ); */

	return result;
    }

    if ( 0 == Vmod_major )
    {
	Vmod_major = result;
    }

    printk( MOD_NAME ": Got major %d\n", Vmod_major );
    return TRUE;

}  /* device_init */


BOOL device_close( void )
{
    int result;

    result = unregister_chrdev( Vmod_major, MOD_NAME );
    printk( "unregister_chrdev( %d, \"" MOD_NAME "\" ) returned %d.\n",
	    Vmod_major, result );

    return TRUE;

}  /* device_close */


static int Vmod_open( struct inode *ip, struct file *fp )
{
    int modem_type = MINOR(fp->f_dentry->d_inode->i_rdev) & 0x7f;
    int modem_num = MINOR(fp->f_dentry->d_inode->i_rdev) >> 7;
    int modem_idx = VMODEM_INDEX( modem_type, modem_num );

    printk( "opening minor %d(%d), fp=0x%08x, ip=0x%08x\n",
	    MINOR(ip->i_rdev), modem_idx, (int) fp, (int) ip );

    if ( 34 != modem_type )
    {
	printk( "Only V34 currently supported\n" );
	return -EPROTONOSUPPORT;
    }

    if ( NULL != Vmodems[modem_idx].modem )
    {
	printk( "Error: modem %d already open!!!\n", modem_num );
	return -EBUSY;
    }

    if ( FALSE == V34_GetModem(&Vmodems[modem_idx].modem, modem_idx) )
    {
	printk( "Error: modem %d allocation failed!\n", modem_num );
	return -ENOMEM;
    }

//    MOD_INC_USE_COUNT;

    Vmod_fops.read = V34mod_read;
    Vmod_fops.write = V34mod_write;
    Vmod_fops.ioctl = V34mod_ioctl;

    return 0;

}  /* Vmod_open */


static int Vmod_close( struct inode *ip, struct file *fp )
{
    int modem_type = MINOR(fp->f_dentry->d_inode->i_rdev) & 0x7f;
    int modem_num = MINOR(fp->f_dentry->d_inode->i_rdev) >> 7;
    int modem_idx = VMODEM_INDEX( modem_type, modem_num );

    printk( "closing minor %d(%d), fp=0x%08x, ip=0x%08x\n",
	    MINOR(ip->i_rdev), modem_idx, (int) fp, (int) ip );

    if ( 34 != modem_type )
    {
	printk( "Only V34 currently supported\n" );
	return -EPROTONOSUPPORT;
    }

    if ( NULL == Vmodems[modem_idx].modem )
    {
	printk( "Error: modem %d not open!!!\n", modem_num );
	return -ENOENT;
    }

    V34_CloseModem( &Vmodems[modem_idx].modem, modem_idx );
    Vmodems[modem_idx].modem_ver = 0;
    Vmodems[modem_idx].globeng_ver = 0;

//    MOD_DEC_USE_COUNT;

    return 0;

}  /* Vmod_close */


static ssize_t V34mod_read( struct file *fp,
			    char *b, size_t nb, loff_t *fo )
{
    printk( "read, fp=0x%08x\n", (int) fp );

    return nb;

}  /* V34mod_read */


static ssize_t V34mod_write( struct file *fp,
			     const char *b, size_t nb, loff_t *fo )
{
    printk( "write, fp=0x%08x\n", (int) fp );

    return nb;

}  /* V34mod_write */


static int V34mod_ioctl( struct inode *ip, struct file *fp,
			 unsigned int cmd, unsigned long arg )
{
    int modem_type = MINOR(fp->f_dentry->d_inode->i_rdev) & 0x7f;
    int modem_num = MINOR(fp->f_dentry->d_inode->i_rdev) >> 7;
    int modem_idx = VMODEM_INDEX( modem_type, modem_num );
    int size = _IOC_SIZE(cmd);

    DWORD dw_ioc_val;
/*      char ioc_buffer[128], tmp_buffer[128]; */

    printk( "ioctl on %d, fp=0x%lx, ip=0x%lx: 0x%08x(%c,%d,%d), arg=0x%lx\n",
	    MINOR(ip->i_rdev), (long int) fp, (long int) ip,
	    cmd, _IOC_TYPE(cmd), _IOC_NR(cmd), size, arg );

    if ( _IOC_TYPE(cmd) != VMOD_IOC_MAGIC )
	return -EINVAL;
    if ( _IOC_NR(cmd) > VMOD_IOC_MAXNR )
	return -EINVAL;

    /* No more need to check user space - done automotically when using */
    /* get_user, put_user, copy_from_user and copy_to_user (and *_ret). */

    switch ( cmd )
    {

    case VMOD_IOC_GETMODVER:
	get_user_ret( dw_ioc_val, (int *) arg, -EFAULT );
	dw_ioc_val = V34_GetCModemVersion();
	put_user_ret( dw_ioc_val, (int *) arg, -EFAULT );
	Vmodems[modem_idx].modem_ver = dw_ioc_val;
	return OK;

    case VMOD_IOC_GETGLBVER:
	dw_ioc_val = V34_GetCAppGlobengVersion();
	put_user_ret( dw_ioc_val, (int *) arg, -EFAULT );
	Vmodems[modem_idx].globeng_ver = dw_ioc_val;
	return OK;

    case VMOD_IOC_MONITOR:
	
	break;

    case VMOD_IOC_CONFIG:
	
	break;

    case VMOD_IOC_SETMODE:
	printk( "SetMode: %ld\n", arg );
	return V34_SetCallMode( Vmodems[modem_idx].modem,
				(DP_MODE_t) arg ) ? 0 : -EINVAL;

    case VMOD_IOC_INITMODEM:
	return V34_InitModem( Vmodems[modem_idx].modem ) ? 0 : -EINVAL;

    case VMOD_IOC_SETBITS:
	
	break;

    case VMOD_IOC_GETBITS:
	
	break;

    case VMOD_IOC_GETEVENTS:
	
	break;

    default:
	return -EINVAL;

    }

    return -EINVAL;

}  /* V34mod_ioctl */
