/*
 * v34cwrapper.cpp - wrpper for CMODEM functions of V34.
 */

#include "ossysenv.h"
#include "globeng.h"
#include "v34wrapper.h"
#include "missing_c.h"
#include "ostime_ex.h"

DP_EXPORT_FUNCTIONS_T V34ExportedFunctions = {
    GetModemInstance, 
    CloseModemInstance, 
    GetCModemVersion, 
    GetCAppGlobengVersion, 
    SetMemoryAllocator, 
};

CGlobals *pGlobeng;

#define NUM_KEPT_EVENTS  16
#define NUM_BYTES_IN_OUT 1024

typedef struct {
    HANDLE modem;
    BYTE *bytes_in;
    int bytes_in_read;
    int bytes_in_write;
    BYTE *bytes_out;
    int bytes_out_read;
    int bytes_out_write;
    EVENT *event_list;
    int event_list_read;
    int event_list_write;
} Vmodem_mng_t;

Vmodem_mng_t Vmodem_mng[2];
const static Vmodem_mng_t mod_clr_mng = {
    NULL, NULL, 0, 0, NULL, 0, 0, {0,}, 0, 0
};

#define PUT_BYTES(src, len, buf, pbufw) do									  \
{																			  \
    if ( (len) > NUM_BYTES_IN_OUT )											  \
    {																		  \
		printk( "ERROR: too much data to put!\n" );							  \
    }																		  \
    else																	  \
    {																		  \
		int __cpylen = min( len, NUM_BYTES_IN_OUT - (*(pbufw)) );			  \
		memcpy( (src), &((buf)[*(pbufw)]), __cpylen );						  \
		*(pbufw) += __cpylen;												  \
		if ( NUM_BYTES_IN_OUT == *(pbufw) )									  \
		{																	  \
			int __cpy2len = (len) - __cpylen;								  \
			memcpy( &((src)[__cpylen]), (buf), __cpy2len );					  \
			*(pbufw) = __cpy2len;											  \
		}																	  \
    }																		  \
} while(0)

#define GET_BYTES(buf, pbufr, dst, len) do									  \
{																			  \
    if ( (len) > NUM_BYTES_IN_OUT )											  \
    {																		  \
		printk( "ERROR: too much data to get!\n" );							  \
    }																		  \
    else																	  \
    {																		  \
		int __cpylen = min( len, NUM_BYTES_IN_OUT - (*(pbufr)) );			  \
		memcpy( &((buf)[*(pbufr)]), (dst), __cpylen );						  \
		*(pbufr) += __cpylen;												  \
		if ( NUM_BYTES_IN_OUT == *(pbufr) )									  \
		{																	  \
			int __cpy2len = (len) - __cpylen;								  \
			memcpy( (buf), &((dst)[__cpylen]), __cpy2len );					  \
			*(pbufr) = __cpy2len;											  \
		}																	  \
    }																		  \
} while(0)

#define PUT_EVENT(evt, evt_buf, evt_but_ptr) do								  \
{																			  \
} while(0)


static void V34BitSource( DPID DataPumpId, PVOID pDataRefData,
			  BYTE* pByteStream, int StreamLength );
static void V34BitTarget( DPID DataPumpId, PVOID pDataRefData,
			  BYTE* pByteStream, int StreamLength );
static void V34EventProc( EVENT *Event );


BOOL Vmod_InitGlobals( void )
{
	int Id;

    Id = OSFloatPrefix();

    pGlobeng = new CGlobals;
    //    g_GlobengRef = new CGlobals;

    if ( (NULL == pGlobeng) || (FALSE == pGlobeng->Init()) )
    {
        printk( "Globals allocation or init Failed!\n" );
        if ( NULL != pGlobeng )
            delete pGlobeng;

    	OSFloatSuffix( Id );
        return FALSE;
    }

#if (OCT_MEM_MGR == 0)
    SetMemoryAllocator( pGlobeng->GetMemMgrPtr() );
#endif

	OSFloatSuffix( Id );
    return TRUE;

}  /* Vmod_InitGlobals */


BOOL Vmod_CloseGlobals( void )
{
	int Id;

	Id = OSFloatPrefix();

    if ( NULL != pGlobeng )
    {
	pGlobeng->Close();
        delete pGlobeng;
        printk( "Globals closed and deleted.\n" );
    }

	OSFloatSuffix( Id );
    return TRUE;

}  /* Vmod_CloseGlobals */


DWORD V34_GetCModemVersion( void )
{
	int Id;
    DWORD ret;

	Id = OSFloatPrefix();

    ret = GetCModemVersion();

	OSFloatSuffix( Id );
    return ret;

}  /* V34_GetCModemVersion */


DWORD V34_GetCAppGlobengVersion( void )
{
	int Id;
    DWORD ret;

	Id = OSFloatPrefix();

    ret = GetCAppGlobengVersion();

	OSFloatSuffix( Id );
    return ret;

}  /* V34_GetCAppGlobengVersion */


BOOL V34_GetModem( HANDLE *modem, int modem_num )
{
    CModem *pmodem;
	int Id;

	Id = OSFloatPrefix();

    pmodem = GetModemInstance( DP_V34 );
    printk( "GetModemInstance returned modem @0x%08lx!\n", (DWORD) pmodem );

    *modem = (HANDLE) pmodem;
    if ( NULL == *modem )
    {
		OSFloatSuffix( Id );
        return FALSE;
    }

    pmodem->SetGlobalsPtr( pGlobeng );
    pmodem->SetReadCallBack ( V34BitSource, (PVOID) modem_num );
    pmodem->SetWriteCallBack( V34BitTarget, (PVOID) modem_num );
    pmodem->SetEventCallback( V34EventProc, (PVOID) modem_num );

    Vmodem_mng[modem_num] = mod_clr_mng;
    Vmodem_mng[modem_num].modem = *modem;

	OSFloatSuffix( Id );
    return TRUE;

}  /* V34_get_modem */


BOOL
V34_ModMonitor( HANDLE modem,
                DWORD dwMonitorCode, PVOID pInBuffer, DWORD nInBufferSize,
                PVOID pOutBuffer, DWORD nOutBufferSize, PDWORD pBytesReturned )
{
    CModem *pmodem = (CModem *) modem;
	int Id;
    BOOL ret;

	Id = OSFloatPrefix();

    ret = pmodem->ModemMonitor( dwMonitorCode, pInBuffer, nInBufferSize,
				pOutBuffer, nOutBufferSize, pBytesReturned );

	OSFloatSuffix( Id );
    return ret;

}  /* V34_ModMonitor */


BOOL
V34_ModConfig( HANDLE modem,
               DWORD dwConfigCode, PVOID pInBuffer, DWORD nInBufferSize,
               PVOID pOutBuffer, DWORD nOutBufferSize, PDWORD pBytesReturned )
{
    CModem *pmodem = (CModem *) modem;
	int Id;
    BOOL ret;

	Id = OSFloatPrefix();

    ret = pmodem->ModemConfigure( dwConfigCode, pInBuffer, nInBufferSize,
				  pOutBuffer, nOutBufferSize, pBytesReturned);

	OSFloatSuffix( Id );
    return ret;

}  /* V34_ModConfig */


BOOL V34_SetCallMode( HANDLE modem, DP_MODE_t mode )
{
    CModem *pmodem = (CModem *) modem;
	int Id;
    BOOL ret = FALSE;

	Id = OSFloatPrefix();

    switch ( mode )
    {

    case ANSWER:
    case CALLER:
	pmodem->SetCallMode( mode );
	ret = TRUE;
	break;

    default:
	break;

    }

	OSFloatSuffix( Id );
    return ret;

}  /* V34_SetCallMode */


BOOL V34_InitModem( HANDLE modem )
{
    CModem *pmodem = (CModem *) modem;
	int Id;
    BOOL ret;

	Id = OSFloatPrefix();

    ret = pmodem->Init();

	OSFloatSuffix( Id );
    return ret;

}  /* V34_InitModem */


BOOL
V34_Apply( HANDLE modem,
           short* InBuffer, int LenInBuf, short* OutBuffer, int LenOutBuf )
{
    CModem *pmodem = (CModem *) modem;
	int Id;
    BOOL ret;

	Id = OSFloatPrefix();

    ret = pmodem->Apply( InBuffer, LenInBuf, OutBuffer, LenOutBuf );

	OSFloatSuffix( Id );
    return ret;

}  /* V34_Apply */


static void V34BitSource( DPID DataPumpId, PVOID pDataRefData,
			  BYTE* pByteStream, int StreamLength )
{
    

}  /* V34BitSource */


static void V34BitTarget( DPID DataPumpId, PVOID pDataRefData,
			  BYTE* pByteStream, int StreamLength )
{
    

}  /* V34BitTarget */


static void V34EventProc( EVENT *Event )
{
    CModem *pmodem;
    int modem_num = (int) Event->pRefData;

    if ( NULL == Vmodem_mng[modem_num].modem )
    {
	printk( "ERROR: Got event %d for NULL modem %d!\n",
		Event->EventType, modem_num );
	return;
    }

    pmodem = (CModem *) Vmodem_mng[modem_num].modem;

    printk( "Got event: %s", pmodem->GetEventName(Event->EventType) );

//      Vmodem_mng[modem_num].event_list[Vmodem_mng[modem_num].event_list_write] =
//  	EVENT;

}  /* V34EventProc */


BOOL V34_CloseModem( HANDLE *modem, int modem_num )
{
    CModem *pmodem = (CModem *) *modem;
	int Id;

	Id = OSFloatPrefix();

    printk( "Closing modem @0x%08lx!\n", (DWORD) pmodem );

    if ( NULL == modem )
    {
		OSFloatSuffix( Id );
		return FALSE;
    }

    pmodem->Close();
    CloseModemInstance( pmodem, DP_V34 );
    *modem = NULL;

    Vmodem_mng[modem_num] = mod_clr_mng;

	OSFloatSuffix( Id );
    return TRUE;

}  /* V34_CloseModem */
